@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2009-2017 Oracle Corporation
+ * Copyright (C) 2009-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -39,6 +49,7 @@
 #include <iprt/system.h>
 #include <iprt/time.h>
 #include <iprt/thread.h>
+#include <iprt/utf16.h>
 
 #include <VBox/VBoxGuestLib.h>
 #include "VBoxServiceInternal.h"
@@ -46,12 +57,6 @@
 #include "VBoxServiceVMInfo.h"
 #include "../../WINNT/VBoxTray/VBoxTrayMsg.h" /* For IPC. */
 
-static uint32_t s_uDebugGuestPropClientID = 0;
-static uint32_t s_uDebugIter = 0;
-/** Whether to skip the logged-in user detection over RDP or not.
- *  See notes in this section why we might want to skip this. */
-static bool s_fSkipRDPDetection = false;
-
 
 /*********************************************************************************************************************************
 *   Structures and Typedefs                                                                                                      *
@@ -59,9 +64,9 @@ static bool s_fSkipRDPDetection = false;
 /** Structure for storing the looked up user information. */
 typedef struct VBOXSERVICEVMINFOUSER
 {
-    WCHAR wszUser[_MAX_PATH];
-    WCHAR wszAuthenticationPackage[_MAX_PATH];
-    WCHAR wszLogonDomain[_MAX_PATH];
+    WCHAR wszUser[MAX_PATH];
+    WCHAR wszAuthenticationPackage[MAX_PATH];
+    WCHAR wszLogonDomain[MAX_PATH];
     /** Number of assigned user processes. */
     ULONG ulNumProcs;
     /** Last (highest) session ID. This
@@ -105,6 +110,12 @@ static int  vgsvcVMInfoWinWriteLastInput(PVBOXSERVICEVEPROPCACHE pCache, const c
 /*********************************************************************************************************************************
 *   Global Variables                                                                                                             *
 *********************************************************************************************************************************/
+static uint32_t s_uDebugGuestPropClientID = 0;
+static uint32_t s_uDebugIter = 0;
+/** Whether to skip the logged-in user detection over RDP or not.
+ *  See notes in this section why we might want to skip this. */
+static bool s_fSkipRDPDetection = false;
+
 static RTONCE                                   g_vgsvcWinVmInitOnce = RTONCE_INITIALIZER;
 
 /** @name Secur32.dll imports are dynamically resolved because of NT4.
@@ -132,9 +143,6 @@ static decltype(QueryFullProcessImageNameW)    *g_pfnQueryFullProcessImageNameW
 
 /** @} */
 
-/** Windows version.  */
-static OSVERSIONINFOEXA                         g_WinVersion;
-
 
 /**
  * An RTOnce callback function.
@@ -145,7 +153,7 @@ static DECLCALLBACK(int) vgsvcWinVmInfoInitOnce(void *pvIgnored)
 
     /* SECUR32 */
     RTLDRMOD hLdrMod;
-    int rc = RTLdrLoadSystem("secur32.dll", true, &hLdrMod);
+    int rc = RTLdrLoadSystem("secur32.dll", true /*fNoUnload*/, &hLdrMod);
     if (RT_SUCCESS(rc))
     {
         rc = RTLdrGetSymbol(hLdrMod, "LsaGetLogonSessionData", (void **)&g_pfnLsaGetLogonSessionData);
@@ -158,15 +166,15 @@ static DECLCALLBACK(int) vgsvcWinVmInfoInitOnce(void *pvIgnored)
     }
     if (RT_FAILURE(rc))
     {
-        VGSvcVerbose(1, "Secur32.dll APIs are not availble (%Rrc)\n", rc);
+        VGSvcVerbose(1, "Secur32.dll APIs are not available (%Rrc)\n", rc);
         g_pfnLsaGetLogonSessionData = NULL;
         g_pfnLsaEnumerateLogonSessions = NULL;
         g_pfnLsaFreeReturnBuffer = NULL;
-        Assert(g_WinVersion.dwMajorVersion < 5);
+        Assert(RTSystemGetNtVersion() < RTSYSTEM_MAKE_NT_VERSION(5, 0, 0));
     }
 
     /* WTSAPI32 */
-    rc = RTLdrLoadSystem("wtsapi32.dll", true, &hLdrMod);
+    rc = RTLdrLoadSystem("wtsapi32.dll", true /*fNoUnload*/, &hLdrMod);
     if (RT_SUCCESS(rc))
     {
         rc = RTLdrGetSymbol(hLdrMod, "WTSFreeMemory", (void **)&g_pfnWTSFreeMemory);
@@ -177,14 +185,14 @@ static DECLCALLBACK(int) vgsvcWinVmInfoInitOnce(void *pvIgnored)
     }
     if (RT_FAILURE(rc))
     {
-        VGSvcVerbose(1, "WtsApi32.dll APIs are not availble (%Rrc)\n", rc);
+        VGSvcVerbose(1, "WtsApi32.dll APIs are not available (%Rrc)\n", rc);
         g_pfnWTSFreeMemory = NULL;
         g_pfnWTSQuerySessionInformationA = NULL;
-        Assert(g_WinVersion.dwMajorVersion < 5);
+        Assert(RTSystemGetNtVersion() < RTSYSTEM_MAKE_NT_VERSION(5, 0, 0));
     }
 
     /* PSAPI */
-    rc = RTLdrLoadSystem("psapi.dll", true, &hLdrMod);
+    rc = RTLdrLoadSystem("psapi.dll", true /*fNoUnload*/, &hLdrMod);
     if (RT_SUCCESS(rc))
     {
         rc = RTLdrGetSymbol(hLdrMod, "EnumProcesses", (void **)&g_pfnEnumProcesses);
@@ -195,44 +203,30 @@ static DECLCALLBACK(int) vgsvcWinVmInfoInitOnce(void *pvIgnored)
     }
     if (RT_FAILURE(rc))
     {
-        VGSvcVerbose(1, "psapi.dll APIs are not availble (%Rrc)\n", rc);
+        VGSvcVerbose(1, "psapi.dll APIs are not available (%Rrc)\n", rc);
         g_pfnEnumProcesses = NULL;
         g_pfnGetModuleFileNameExW = NULL;
-        Assert(g_WinVersion.dwMajorVersion < 5);
+        Assert(RTSystemGetNtVersion() < RTSYSTEM_MAKE_NT_VERSION(5, 0, 0));
     }
 
     /* Kernel32: */
-    rc = RTLdrLoadSystem("kernel32.dll", true, &hLdrMod);
+    rc = RTLdrLoadSystem("kernel32.dll", true /*fNoUnload*/, &hLdrMod);
     AssertRCReturn(rc, rc);
     rc = RTLdrGetSymbol(hLdrMod, "QueryFullProcessImageNameW", (void **)&g_pfnQueryFullProcessImageNameW);
     if (RT_FAILURE(rc))
     {
-        Assert(g_WinVersion.dwMajorVersion < 6);
+        Assert(RTSystemGetNtVersion() < RTSYSTEM_MAKE_NT_VERSION(6, 0, 0));
         g_pfnQueryFullProcessImageNameW = NULL;
     }
     RTLdrClose(hLdrMod);
 
-    /*
-     * Get the extended windows version once and for all.
-     */
-    g_WinVersion.dwOSVersionInfoSize = sizeof(g_WinVersion);
-    if (!GetVersionExA((OSVERSIONINFO *)&g_WinVersion))
-    {
-        AssertFailed();
-        RT_ZERO(g_WinVersion);
-        g_WinVersion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-        if (!GetVersionExA((OSVERSIONINFO *)&g_WinVersion))
-            RT_ZERO(g_WinVersion);
-    }
-
     return VINF_SUCCESS;
 }
 
 
 static bool vgsvcVMInfoSession0Separation(void)
 {
-    return g_WinVersion.dwPlatformId == VER_PLATFORM_WIN32_NT
-        && g_WinVersion.dwMajorVersion >= 6; /* Vista = 6.0 */
+    return RTSystemGetNtVersion() >= RTSYSTEM_MAKE_NT_VERSION(6, 0, 0); /* Vista */
 }
 
 
@@ -241,61 +235,57 @@ static bool vgsvcVMInfoSession0Separation(void)
  *
  * @return  IPRT status code.
  */
-static int vgsvcVMInfoWinProcessesGetModuleNameA(PVBOXSERVICEVMINFOPROC const pProc, PRTUTF16 *ppszName)
+static int vgsvcVMInfoWinProcessesGetModuleNameW(PVBOXSERVICEVMINFOPROC const pProc, PRTUTF16 *ppszName)
 {
-    AssertPtrReturn(pProc, VERR_INVALID_POINTER);
+    *ppszName = NULL;
     AssertPtrReturn(ppszName, VERR_INVALID_POINTER);
+    AssertPtrReturn(pProc, VERR_INVALID_POINTER);
+    AssertReturn(g_pfnGetModuleFileNameExW || g_pfnQueryFullProcessImageNameW, VERR_NOT_SUPPORTED);
 
-    /** @todo Only do this once. Later. */
-    /* Platform other than NT (e.g. Win9x) not supported. */
-    if (g_WinVersion.dwPlatformId != VER_PLATFORM_WIN32_NT)
-        return VERR_NOT_SUPPORTED;
-
-    int rc = VINF_SUCCESS;
-
+    /*
+     * Open the process.
+     */
     DWORD dwFlags = PROCESS_QUERY_INFORMATION | PROCESS_VM_READ;
-    if (g_WinVersion.dwMajorVersion >= 6 /* Vista or later */)
+    if (RTSystemGetNtVersion() >= RTSYSTEM_MAKE_NT_VERSION(6, 0, 0)) /* Vista and later */
         dwFlags = PROCESS_QUERY_LIMITED_INFORMATION; /* possible to do on more processes */
 
-    HANDLE h = OpenProcess(dwFlags, FALSE, pProc->id);
-    if (h == NULL)
+    HANDLE hProcess = OpenProcess(dwFlags, FALSE, pProc->id);
+    if (hProcess == NULL)
     {
         DWORD dwErr = GetLastError();
         if (g_cVerbosity)
             VGSvcError("Unable to open process with PID=%u, error=%u\n", pProc->id, dwErr);
-        rc = RTErrConvertFromWin32(dwErr);
+        return RTErrConvertFromWin32(dwErr);
     }
+
+    /*
+     * Since GetModuleFileNameEx has trouble with cross-bitness stuff (32-bit apps
+     * cannot query 64-bit apps and vice verse) we have to use a different code
+     * path for Vista and up.
+     *
+     * So use QueryFullProcessImageNameW when available (Vista+), fall back on
+     * GetModuleFileNameExW on older windows version (
+     */
+    WCHAR wszName[_1K];
+    DWORD dwLen = _1K;
+    BOOL  fRc;
+    if (g_pfnQueryFullProcessImageNameW)
+        fRc = g_pfnQueryFullProcessImageNameW(hProcess, 0 /*PROCESS_NAME_NATIVE*/, wszName, &dwLen);
+    else
+        fRc = g_pfnGetModuleFileNameExW(hProcess, NULL /* Get main executable */, wszName, dwLen);
+
+    int rc;
+    if (fRc)
+        rc = RTUtf16DupEx(ppszName, wszName, 0);
     else
     {
-        /* Since GetModuleFileNameEx has trouble with cross-bitness stuff (32-bit apps cannot query 64-bit
-           apps and vice verse) we have to use a different code path for Vista and up. */
-        WCHAR wszName[_1K];
-        DWORD dwLen = sizeof(wszName); /** @todo r=bird: wrong? */
-
-        /* Use QueryFullProcessImageNameW if available (Vista+). */
-        if (g_pfnQueryFullProcessImageNameW)
-        {
-            if (!g_pfnQueryFullProcessImageNameW(h, 0 /*PROCESS_NAME_NATIVE*/, wszName, &dwLen))
-                rc = VERR_ACCESS_DENIED;
-        }
-        else if (!g_pfnGetModuleFileNameExW(h, NULL /* Get main executable */, wszName, dwLen))
-            rc = VERR_ACCESS_DENIED;
-
-        if (   RT_FAILURE(rc)
-            && g_cVerbosity > 3)
-           VGSvcError("Unable to retrieve process name for PID=%u, error=%u\n", pProc->id, GetLastError());
-        else
-        {
-            PRTUTF16 pszName = RTUtf16Dup(wszName);
-            if (pszName)
-                *ppszName = pszName;
-            else
-                rc = VERR_NO_MEMORY;
-        }
-
-        CloseHandle(h);
+        DWORD dwErr = GetLastError();
+        if (g_cVerbosity > 3)
+            VGSvcError("Unable to retrieve process name for PID=%u, LastError=%Rwc\n", pProc->id, dwErr);
+        rc = RTErrConvertFromWin32(dwErr);
     }
 
+    CloseHandle(hProcess);
     return rc;
 }
 
@@ -683,7 +673,7 @@ static uint32_t vgsvcVMInfoWinSessionHasProcesses(PLUID pSession, PVBOXSERVICEVM
                 if (g_cVerbosity)
                 {
                     PRTUTF16 pszName;
-                    int rc2 = vgsvcVMInfoWinProcessesGetModuleNameA(&paProcs[i], &pszName);
+                    int rc2 = vgsvcVMInfoWinProcessesGetModuleNameW(&paProcs[i], &pszName);
                     VGSvcVerbose(4, "Session %RU32: PID=%u (fInt=%RTbool): %ls\n",
                                  pSessionData->Session, paProcs[i].id, paProcs[i].fInteractive,
                                  RT_SUCCESS(rc2) ? pszName : L"<Unknown>");
@@ -746,14 +736,15 @@ static bool vgsvcVMInfoWinIsLoggedIn(PVBOXSERVICEVMINFOUSER pUserInfo, PLUID pSe
     AssertPtrReturn(pUserInfo, false);
     if (!pSession)
         return false;
-    if (!g_pfnLsaGetLogonSessionData)
+    if (   !g_pfnLsaGetLogonSessionData
+        || !g_pfnLsaNtStatusToWinError)
         return false;
 
     PSECURITY_LOGON_SESSION_DATA pSessionData = NULL;
     NTSTATUS rcNt = g_pfnLsaGetLogonSessionData(pSession, &pSessionData);
     if (rcNt != STATUS_SUCCESS)
     {
-        ULONG ulError = LsaNtStatusToWinError(rcNt);
+        ULONG ulError = g_pfnLsaNtStatusToWinError(rcNt);
         switch (ulError)
         {
             case ERROR_NOT_ENOUGH_MEMORY:
@@ -821,9 +812,9 @@ static bool vgsvcVMInfoWinIsLoggedIn(PVBOXSERVICEVMINFOUSER pUserInfo, PLUID pSe
         vgsvcVMInfoWinSafeCopy(pUserInfo->wszLogonDomain, sizeof(pUserInfo->wszLogonDomain),
                                &pSessionData->LogonDomain, "Logon domain name");
 
-        TCHAR           szOwnerName[_MAX_PATH]  = { 0 };
+        TCHAR           szOwnerName[MAX_PATH]   = { 0 };
         DWORD           dwOwnerNameSize         = sizeof(szOwnerName);
-        TCHAR           szDomainName[_MAX_PATH] = { 0 };
+        TCHAR           szDomainName[MAX_PATH]  = { 0 };
         DWORD           dwDomainNameSize        = sizeof(szDomainName);
         SID_NAME_USE    enmOwnerType            = SidTypeInvalid;
         if (!LookupAccountSid(NULL,
@@ -852,29 +843,30 @@ static bool vgsvcVMInfoWinIsLoggedIn(PVBOXSERVICEVMINFOUSER pUserInfo, PLUID pSe
                              pUserInfo->wszUser, pSessionData->Session, pSessionData->LogonId.HighPart,
                              pSessionData->LogonId.LowPart, pUserInfo->wszAuthenticationPackage, pUserInfo->wszLogonDomain);
 
-                /**
-                 * Note: On certain Windows OSes WTSQuerySessionInformation leaks memory when used
-                 * under a heavy stress situation. There are hotfixes available from Microsoft.
+                /* KB970910 (check http://support.microsoft.com/kb/970910 on archive.org)
+                 * indicates that WTSQuerySessionInformation may leak memory and return the
+                 * wrong status code for WTSApplicationName and WTSInitialProgram queries.
                  *
-                 * See: http://support.microsoft.com/kb/970910
+                 * The system must be low on resources, and presumably some internal operation
+                 * must fail because of this, triggering an error handling path that forgets
+                 * to free memory and set last error.
+                 *
+                 * bird 2022-08-26: However, we do not query either of those info items.  We
+                 * query WTSConnectState, which is a rather simple affair.  So, I've
+                 * re-enabled the code for all systems that includes the API.
                  */
                 if (!s_fSkipRDPDetection)
                 {
-                    /* Skip RDP detection on non-NT systems. */
-                    if (g_WinVersion.dwPlatformId != VER_PLATFORM_WIN32_NT)
-                        s_fSkipRDPDetection = true;
-
-                    /* Skip RDP detection on Windows 2000.
-                     * For Windows 2000 however we don't have any hotfixes, so just skip the
-                     * RDP detection in any case. */
-                    if (   g_WinVersion.dwMajorVersion == 5
-                        && g_WinVersion.dwMinorVersion == 0)
-                        s_fSkipRDPDetection = true;
-
                     /* Skip if we don't have the WTS API. */
                     if (!g_pfnWTSQuerySessionInformationA)
                         s_fSkipRDPDetection = true;
-
+#if 0 /* bird: see above */
+                    /* Skip RDP detection on Windows 2000 and older.
+                       For Windows 2000 however we don't have any hotfixes, so just skip the
+                       RDP detection in any case. */
+                    else if (RTSystemGetNtVersion() < RTSYSTEM_MAKE_NT_VERSION(5, 1, 0)) /* older than XP */
+                        s_fSkipRDPDetection = true;
+#endif
                     if (s_fSkipRDPDetection)
                         VGSvcVerbose(0, "Detection of logged-in users via RDP is disabled\n");
                 }
@@ -969,23 +961,22 @@ static int vgsvcVMInfoWinWriteLastInput(PVBOXSERVICEVEPROPCACHE pCache, const ch
             VBOXTRAYIPCHEADER ipcHdr =
             {
                 /* .uMagic      = */ VBOXTRAY_IPC_HDR_MAGIC,
-                /* .uHdrVersion = */ 0,
-                /* .uMsgType    = */ VBOXTRAYIPCMSGTYPE_USERLASTINPUT,
-                /* .cbMsgData   = */ 0 /* No msg */
+                /* .uVersion    = */ VBOXTRAY_IPC_HDR_VERSION,
+                /* .enmMsgType  = */ VBOXTRAYIPCMSGTYPE_USER_LAST_INPUT,
+                /* .cbPayload   = */ 0 /* No payload */
             };
 
             rc = RTLocalIpcSessionWrite(hSession, &ipcHdr, sizeof(ipcHdr));
-
             if (RT_SUCCESS(rc))
             {
-                VBOXTRAYIPCRES_USERLASTINPUT ipcRes;
-                rc = RTLocalIpcSessionRead(hSession, &ipcRes, sizeof(ipcRes), NULL /* Exact read */);
+                VBOXTRAYIPCREPLY_USER_LAST_INPUT_T ipcReply;
+                rc = RTLocalIpcSessionRead(hSession, &ipcReply, sizeof(ipcReply), NULL /* Exact read */);
                 if (   RT_SUCCESS(rc)
                     /* If uLastInput is set to UINT32_MAX VBoxTray was not able to retrieve the
                      * user's last input time. This might happen when running on Windows NT4 or older. */
-                    && ipcRes.uLastInput != UINT32_MAX)
+                    && ipcReply.cSecSinceLastInput != UINT32_MAX)
                 {
-                    userState = (ipcRes.uLastInput * 1000) < g_uVMInfoUserIdleThresholdMS
+                    userState = ipcReply.cSecSinceLastInput * 1000 < g_uVMInfoUserIdleThresholdMS
                               ? VBoxGuestUserState_InUse
                               : VBoxGuestUserState_Idle;
 
@@ -999,12 +990,12 @@ static int vgsvcVMInfoWinWriteLastInput(PVBOXSERVICEVEPROPCACHE pCache, const ch
                      */
                     fReportToHost = rc == VINF_SUCCESS;
                     VGSvcVerbose(4, "User '%s' (domain '%s') is idle for %RU32, fReportToHost=%RTbool\n",
-                                 pszUser, pszDomain ? pszDomain : "<None>", ipcRes.uLastInput, fReportToHost);
+                                 pszUser, pszDomain ? pszDomain : "<None>", ipcReply.cSecSinceLastInput, fReportToHost);
 
 #if 0 /* Do we want to write the idle time as well? */
                         /* Also write the user's current idle time, if there is any. */
                         if (userState == VBoxGuestUserState_Idle)
-                            rc = vgsvcUserUpdateF(pCache, pszUser, pszDomain, "IdleTimeMs", "%RU32", ipcRes.uLastInputMs);
+                            rc = vgsvcUserUpdateF(pCache, pszUser, pszDomain, "IdleTimeMs", "%RU32", ipcReply.cSecSinceLastInput);
                         else
                             rc = vgsvcUserUpdateF(pCache, pszUser, pszDomain, "IdleTimeMs", NULL /* Delete property */);
 
@@ -1012,7 +1003,7 @@ static int vgsvcVMInfoWinWriteLastInput(PVBOXSERVICEVEPROPCACHE pCache, const ch
 #endif
                 }
 #ifdef DEBUG
-                else if (RT_SUCCESS(rc) && ipcRes.uLastInput == UINT32_MAX)
+                else if (RT_SUCCESS(rc) && ipcReply.cSecSinceLastInput == UINT32_MAX)
                     VGSvcVerbose(4, "Last input for user '%s' is not supported, skipping\n", pszUser, rc);
 #endif
             }
@@ -1083,7 +1074,7 @@ int VGSvcVMInfoWinWriteUsers(PVBOXSERVICEVEPROPCACHE pCache, char **ppszUserList
     int rc = RTOnce(&g_vgsvcWinVmInitOnce, vgsvcWinVmInfoInitOnce, NULL);
     if (RT_FAILURE(rc))
         return rc;
-    if (!g_pfnLsaEnumerateLogonSessions || !g_pfnEnumProcesses)
+    if (!g_pfnLsaEnumerateLogonSessions || !g_pfnEnumProcesses || !g_pfnLsaNtStatusToWinError)
         return VERR_NOT_SUPPORTED;
 
     rc = VbglR3GuestPropConnect(&s_uDebugGuestPropClientID);
@@ -1099,7 +1090,7 @@ int VGSvcVMInfoWinWriteUsers(PVBOXSERVICEVEPROPCACHE pCache, char **ppszUserList
     NTSTATUS rcNt = g_pfnLsaEnumerateLogonSessions(&cSessions, &paSessions);
     if (rcNt != STATUS_SUCCESS)
     {
-        ULONG uError = LsaNtStatusToWinError(rcNt);
+        ULONG uError = g_pfnLsaNtStatusToWinError(rcNt);
         switch (uError)
         {
             case ERROR_NOT_ENOUGH_MEMORY:
@@ -1183,9 +1174,9 @@ int VGSvcVMInfoWinWriteUsers(PVBOXSERVICEVEPROPCACHE pCache, char **ppszUserList
                         PVBOXSERVICEVMINFOUSER pCurUser = &pUserInfo[a];
                         AssertPtr(pCurUser);
 
-                        if (   !wcscmp(userSession.wszUser, pCurUser->wszUser)
-                            && !wcscmp(userSession.wszLogonDomain, pCurUser->wszLogonDomain)
-                            && !wcscmp(userSession.wszAuthenticationPackage, pCurUser->wszAuthenticationPackage))
+                        if (   !RTUtf16Cmp(userSession.wszUser, pCurUser->wszUser)
+                            && !RTUtf16Cmp(userSession.wszLogonDomain, pCurUser->wszLogonDomain)
+                            && !RTUtf16Cmp(userSession.wszAuthenticationPackage, pCurUser->wszAuthenticationPackage))
                         {
                             /*
                              * Only respect the highest session for the current user.
@@ -1314,75 +1305,57 @@ int VGSvcVMInfoWinWriteUsers(PVBOXSERVICEVEPROPCACHE pCache, char **ppszUserList
 int VGSvcVMInfoWinGetComponentVersions(uint32_t uClientID)
 {
     int rc;
-    char szSysDir[_MAX_PATH] = {0};
-    char szWinDir[_MAX_PATH] = {0};
-    char szDriversDir[_MAX_PATH + 32] = {0};
+    char szSysDir[MAX_PATH] = {0};
+    char szWinDir[MAX_PATH] = {0};
+    char szDriversDir[MAX_PATH + 32] = {0};
 
     /* ASSUME: szSysDir and szWinDir and derivatives are always ASCII compatible. */
-    GetSystemDirectory(szSysDir, _MAX_PATH);
-    GetWindowsDirectory(szWinDir, _MAX_PATH);
+    GetSystemDirectory(szSysDir, MAX_PATH);
+    GetWindowsDirectory(szWinDir, MAX_PATH);
     RTStrPrintf(szDriversDir, sizeof(szDriversDir), "%s\\drivers", szSysDir);
 #ifdef RT_ARCH_AMD64
-    char szSysWowDir[_MAX_PATH + 32] = {0};
+    char szSysWowDir[MAX_PATH + 32] = {0};
     RTStrPrintf(szSysWowDir, sizeof(szSysWowDir), "%s\\SysWow64", szWinDir);
 #endif
 
     /* The file information table. */
     const VBOXSERVICEVMINFOFILE aVBoxFiles[] =
     {
-        { szSysDir, "VBoxControl.exe" },
-        { szSysDir, "VBoxHook.dll" },
-        { szSysDir, "VBoxDisp.dll" },
-        { szSysDir, "VBoxTray.exe" },
-#ifdef TARGET_NT4
-        { szSysDir, "VBoxServiceNT.exe" },
-#else
-        { szSysDir, "VBoxService.exe" },
-        { szSysDir, "VBoxMRXNP.dll" },
-        { szSysDir, "VBoxGINA.dll" },
-        { szSysDir, "VBoxCredProv.dll" },
-#endif
+        { szSysDir,     "VBoxControl.exe" },
+        { szSysDir,     "VBoxHook.dll" },
+        { szSysDir,     "VBoxDisp.dll" },
+        { szSysDir,     "VBoxTray.exe" },
+        { szSysDir,     "VBoxService.exe" },
+        { szSysDir,     "VBoxMRXNP.dll" },
+        { szSysDir,     "VBoxGINA.dll" },
+        { szSysDir,     "VBoxCredProv.dll" },
 
  /* On 64-bit we don't yet have the OpenGL DLLs in native format.
     So just enumerate the 32-bit files in the SYSWOW directory. */
 #ifdef RT_ARCH_AMD64
-        { szSysWowDir, "VBoxOGLarrayspu.dll" },
-        { szSysWowDir, "VBoxOGLcrutil.dll" },
-        { szSysWowDir, "VBoxOGLerrorspu.dll" },
-        { szSysWowDir, "VBoxOGLpackspu.dll" },
-        { szSysWowDir, "VBoxOGLpassthroughspu.dll" },
-        { szSysWowDir, "VBoxOGLfeedbackspu.dll" },
-        { szSysWowDir, "VBoxOGL.dll" },
+        { szSysWowDir,  "VBoxOGL-x86.dll" },
 #else  /* !RT_ARCH_AMD64 */
-# ifdef TARGET_NT4
-        { szSysDir, "VBoxOGLarrayspu.dll" },
-        { szSysDir, "VBoxOGLcrutil.dll" },
-        { szSysDir, "VBoxOGLerrorspu.dll" },
-        { szSysDir, "VBoxOGLpackspu.dll" },
-        { szSysDir, "VBoxOGLpassthroughspu.dll" },
-        { szSysDir, "VBoxOGLfeedbackspu.dll" },
-        { szSysDir, "VBoxOGL.dll" },
-# endif
+        { szSysDir,     "VBoxOGL.dll" },
 #endif /* !RT_ARCH_AMD64 */
 
-#ifdef TARGET_NT4
-        { szDriversDir, "VBoxGuestNT.sys" },
-        { szDriversDir, "VBoxMouseNT.sys" },
-#else
         { szDriversDir, "VBoxGuest.sys" },
+        { szDriversDir, "VBoxMouseNT.sys" },
         { szDriversDir, "VBoxMouse.sys" },
         { szDriversDir, "VBoxSF.sys"    },
-#endif
         { szDriversDir, "VBoxVideo.sys" },
     };
 
     for (unsigned i = 0; i < RT_ELEMENTS(aVBoxFiles); i++)
     {
         char szVer[128];
-        VGSvcUtilWinGetFileVersionString(aVBoxFiles[i].pszFilePath, aVBoxFiles[i].pszFileName, szVer, sizeof(szVer));
+        rc = VGSvcUtilWinGetFileVersionString(aVBoxFiles[i].pszFilePath, aVBoxFiles[i].pszFileName, szVer, sizeof(szVer));
         char szPropPath[256];
         RTStrPrintf(szPropPath, sizeof(szPropPath), "/VirtualBox/GuestAdd/Components/%s", aVBoxFiles[i].pszFileName);
-        rc = VGSvcWritePropF(uClientID, szPropPath, "%s", szVer);
+        if (   rc != VERR_FILE_NOT_FOUND
+            && rc != VERR_PATH_NOT_FOUND)
+            VGSvcWritePropF(uClientID, szPropPath, "%s", szVer);
+        else
+            VGSvcWritePropF(uClientID, szPropPath, NULL);
     }
 
     return VINF_SUCCESS;
