@@ -1,19 +1,29 @@
 /** @file
   RTC Architectural Protocol GUID as defined in DxeCis 0.96.
 
-Copyright (c) 2006 - 2015, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
+Copyright (c) 2006 - 2018, Intel Corporation. All rights reserved.<BR>
+Copyright (c) 2017, AMD Inc. All rights reserved.<BR>
+Copyright (c) 2018 - 2020, ARM Limited. All rights reserved.<BR>
 
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
 #include "PcRtc.h"
 
+extern UINTN  mRtcIndexRegister;
+extern UINTN  mRtcTargetRegister;
+
+//
+// Days of month.
+//
+UINTN mDayOfMonth[] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
+
+//
+// The name of NV variable to store the timezone and daylight saving information.
+//
+CHAR16 mTimeZoneVariableName[] = L"RTC";
+
 /**
   Compare the Hour, Minute and Second of the From time and the To time.
 
@@ -48,38 +58,132 @@ IsWithinOneDay (
   );
 
 /**
-  Read RTC content through its registers.
+  Read RTC content through its registers using IO access.
 
-  @param  Address  Address offset of RTC. It is recommended to use macros such as
-                   RTC_ADDRESS_SECONDS.
+  @param  Address   Address offset of RTC. It is recommended to use
+                    macros such as RTC_ADDRESS_SECONDS.
 
   @return The data of UINT8 type read from RTC.
 **/
+STATIC
 UINT8
-RtcRead (
-  IN  UINT8 Address
+IoRtcRead (
+  IN  UINTN Address
   )
 {
-  IoWrite8 (PCAT_RTC_ADDRESS_REGISTER, (UINT8) (Address | (UINT8) (IoRead8 (PCAT_RTC_ADDRESS_REGISTER) & 0x80)));
-  return IoRead8 (PCAT_RTC_DATA_REGISTER);
+  IoWrite8 (
+    PcdGet8 (PcdRtcIndexRegister),
+    (UINT8)(Address | (UINT8)(IoRead8 (PcdGet8 (PcdRtcIndexRegister)) & 0x80))
+    );
+  return IoRead8 (PcdGet8 (PcdRtcTargetRegister));
+}
+
+/**
+  Write RTC through its registers  using IO access.
+
+  @param  Address   Address offset of RTC. It is recommended to use
+                    macros such as RTC_ADDRESS_SECONDS.
+  @param  Data      The content you want to write into RTC.
+
+**/
+STATIC
+VOID
+IoRtcWrite (
+  IN  UINTN   Address,
+  IN  UINT8   Data
+  )
+{
+  IoWrite8 (
+    PcdGet8 (PcdRtcIndexRegister),
+    (UINT8)(Address | (UINT8)(IoRead8 (PcdGet8 (PcdRtcIndexRegister)) & 0x80))
+    );
+  IoWrite8 (PcdGet8 (PcdRtcTargetRegister), Data);
+}
+
+/**
+  Read RTC content through its registers using MMIO access.
+
+  @param  Address   Address offset of RTC. It is recommended to use
+                    macros such as RTC_ADDRESS_SECONDS.
+
+  @return The data of UINT8 type read from RTC.
+**/
+STATIC
+UINT8
+MmioRtcRead (
+  IN  UINTN Address
+  )
+{
+  MmioWrite8 (
+    mRtcIndexRegister,
+    (UINT8)(Address | (UINT8)(MmioRead8 (mRtcIndexRegister) & 0x80))
+    );
+  return MmioRead8 (mRtcTargetRegister);
+}
+
+/**
+  Write RTC through its registers using MMIO access.
+
+  @param  Address   Address offset of RTC. It is recommended to use
+                    macros such as RTC_ADDRESS_SECONDS.
+  @param  Data      The content you want to write into RTC.
+
+**/
+STATIC
+VOID
+MmioRtcWrite (
+  IN  UINTN   Address,
+  IN  UINT8   Data
+  )
+{
+  MmioWrite8 (
+    mRtcIndexRegister,
+    (UINT8)(Address | (UINT8)(MmioRead8 (mRtcIndexRegister) & 0x80))
+    );
+  MmioWrite8 (mRtcTargetRegister, Data);
+}
+
+/**
+  Read RTC content through its registers.
+
+  @param  Address   Address offset of RTC. It is recommended to use
+                    macros such as RTC_ADDRESS_SECONDS.
+
+  @return The data of UINT8 type read from RTC.
+**/
+STATIC
+UINT8
+RtcRead (
+  IN  UINTN Address
+  )
+{
+  if (FeaturePcdGet (PcdRtcUseMmio)) {
+    return MmioRtcRead (Address);
+  }
+
+  return IoRtcRead (Address);
 }
 
 /**
   Write RTC through its registers.
 
-  @param  Address  Address offset of RTC. It is recommended to use macros such as
-                   RTC_ADDRESS_SECONDS.
-  @param  Data     The content you want to write into RTC.
+  @param  Address   Address offset of RTC. It is recommended to use
+                    macros such as RTC_ADDRESS_SECONDS.
+  @param  Data      The content you want to write into RTC.
 
 **/
+STATIC
 VOID
 RtcWrite (
-  IN  UINT8   Address,
+  IN  UINTN   Address,
   IN  UINT8   Data
   )
 {
-  IoWrite8 (PCAT_RTC_ADDRESS_REGISTER, (UINT8) (Address | (UINT8) (IoRead8 (PCAT_RTC_ADDRESS_REGISTER) & 0x80)));
-  IoWrite8 (PCAT_RTC_DATA_REGISTER, Data);
+  if (FeaturePcdGet (PcdRtcUseMmio)) {
+    MmioRtcWrite (Address, Data);
+  } else {
+    IoRtcWrite (Address, Data);
+  }
 }
 
 /**
@@ -100,7 +204,6 @@ PcRtcInit (
   RTC_REGISTER_A  RegisterA;
   RTC_REGISTER_B  RegisterB;
   RTC_REGISTER_D  RegisterD;
-  UINT8           Century;
   EFI_TIME        Time;
   UINTN           DataSize;
   UINT32          TimerVar;
@@ -119,7 +222,7 @@ PcRtcInit (
   // Make sure Division Chain is properly configured,
   // or RTC clock won't "tick" -- time won't increment
   //
-  RegisterA.Data = RTC_INIT_REGISTER_A;
+  RegisterA.Data = FixedPcdGet8 (PcdInitialValueRtcRegisterA);
   RtcWrite (RTC_ADDRESS_REGISTER_A, RegisterA.Data);
 
   //
@@ -135,7 +238,7 @@ PcRtcInit (
   //
   // Clear RTC register D
   //
-  RegisterD.Data = RTC_INIT_REGISTER_D;
+  RegisterD.Data = FixedPcdGet8 (PcdInitialValueRtcRegisterD);
   RtcWrite (RTC_ADDRESS_REGISTER_D, RegisterD.Data);
 
   //
@@ -163,13 +266,12 @@ PcRtcInit (
   Time.Month  = RtcRead (RTC_ADDRESS_MONTH);
   Time.Year   = RtcRead (RTC_ADDRESS_YEAR);
 
-  Century = RtcRead (RTC_ADDRESS_CENTURY);
-
   //
   // Set RTC configuration after get original time
   // The value of bit AIE should be reserved.
   //
-  RtcWrite (RTC_ADDRESS_REGISTER_B, (UINT8)(RTC_INIT_REGISTER_B | (RegisterB.Data & BIT5)));
+  RegisterB.Data = FixedPcdGet8 (PcdInitialValueRtcRegisterB) | (RegisterB.Data & BIT5);
+  RtcWrite (RTC_ADDRESS_REGISTER_B, RegisterB.Data);
 
   //
   // Release RTC Lock.
@@ -184,11 +286,11 @@ PcRtcInit (
   //
   DataSize = sizeof (UINT32);
   Status = EfiGetVariable (
-             L"RTC",
+             mTimeZoneVariableName,
              &gEfiCallerIdGuid,
              NULL,
              &DataSize,
-             (VOID *) &TimerVar
+             &TimerVar
              );
   if (!EFI_ERROR (Status)) {
     Time.TimeZone = (INT16) TimerVar;
@@ -201,7 +303,7 @@ PcRtcInit (
   //
   // Validate time fields
   //
-  Status = ConvertRtcTimeToEfiTime (&Time, Century, RegisterB);
+  Status = ConvertRtcTimeToEfiTime (&Time, RegisterB);
   if (!EFI_ERROR (Status)) {
     Status = RtcTimeFieldsValid (&Time);
   }
@@ -218,7 +320,7 @@ PcRtcInit (
     Time.Hour   = RTC_INIT_HOUR;
     Time.Day    = RTC_INIT_DAY;
     Time.Month  = RTC_INIT_MONTH;
-    Time.Year   = RTC_INIT_YEAR;
+    Time.Year   = PcdGet16 (PcdMinimalValidYear);
     Time.Nanosecond  = 0;
     Time.TimeZone = EFI_UNSPECIFIED_TIMEZONE;
     Time.Daylight = 0;
@@ -251,7 +353,7 @@ PcRtcInit (
   Time.Hour   = RTC_INIT_HOUR;
   Time.Day    = RTC_INIT_DAY;
   Time.Month  = RTC_INIT_MONTH;
-  Time.Year   = RTC_INIT_YEAR;
+  Time.Year   = PcdGet16 (PcdMinimalValidYear);
   Time.Nanosecond  = 0;
   Time.TimeZone = Global->SavedTimeZone;
   Time.Daylight = Global->Daylight;;
@@ -273,7 +375,7 @@ PcRtcInit (
     return EFI_DEVICE_ERROR;
   }
 
-  ConvertEfiTimeToRtcTime (&Time, RegisterB, &Century);
+  ConvertEfiTimeToRtcTime (&Time, RegisterB);
 
   //
   // Set the Y/M/D info to variable as it has no corresponding hw registers.
@@ -343,7 +445,6 @@ PcRtcGetTime (
 {
   EFI_STATUS      Status;
   RTC_REGISTER_B  RegisterB;
-  UINT8           Century;
 
   //
   // Check parameters for null pointer
@@ -383,8 +484,6 @@ PcRtcGetTime (
   Time->Month   = RtcRead (RTC_ADDRESS_MONTH);
   Time->Year    = RtcRead (RTC_ADDRESS_YEAR);
 
-  Century = RtcRead (RTC_ADDRESS_CENTURY);
-
   //
   // Release RTC Lock.
   //
@@ -401,7 +500,7 @@ PcRtcGetTime (
   //
   // Make sure all field values are in correct range
   //
-  Status = ConvertRtcTimeToEfiTime (Time, Century, RegisterB);
+  Status = ConvertRtcTimeToEfiTime (Time, RegisterB);
   if (!EFI_ERROR (Status)) {
     Status = RtcTimeFieldsValid (Time);
   }
@@ -447,7 +546,6 @@ PcRtcSetTime (
   EFI_STATUS      Status;
   EFI_TIME        RtcTime;
   RTC_REGISTER_B  RegisterB;
-  UINT8           Century;
   UINT32          TimerVar;
 
   if (Time == NULL) {
@@ -483,15 +581,29 @@ PcRtcSetTime (
   //
   // Write timezone and daylight to RTC variable
   //
-  TimerVar = Time->Daylight;
-  TimerVar = (UINT32) ((TimerVar << 16) | (UINT16)(Time->TimeZone));
-  Status =  EfiSetVariable (
-              L"RTC",
-              &gEfiCallerIdGuid,
-              EFI_VARIABLE_BOOTSERVICE_ACCESS | EFI_VARIABLE_RUNTIME_ACCESS | EFI_VARIABLE_NON_VOLATILE,
-              sizeof (TimerVar),
-              &TimerVar
-              );
+  if ((Time->TimeZone == EFI_UNSPECIFIED_TIMEZONE) && (Time->Daylight == 0)) {
+    Status = EfiSetVariable (
+               mTimeZoneVariableName,
+               &gEfiCallerIdGuid,
+               0,
+               0,
+               NULL
+               );
+    if (Status == EFI_NOT_FOUND) {
+      Status = EFI_SUCCESS;
+    }
+  } else {
+    TimerVar = Time->Daylight;
+    TimerVar = (UINT32) ((TimerVar << 16) | (UINT16)(Time->TimeZone));
+    Status = EfiSetVariable (
+               mTimeZoneVariableName,
+               &gEfiCallerIdGuid,
+               EFI_VARIABLE_BOOTSERVICE_ACCESS | EFI_VARIABLE_RUNTIME_ACCESS | EFI_VARIABLE_NON_VOLATILE,
+               sizeof (TimerVar),
+               &TimerVar
+               );
+  }
+
   if (EFI_ERROR (Status)) {
     if (!EfiAtRuntime ()) {
       EfiReleaseLock (&Global->RtcLock);
@@ -506,7 +618,14 @@ PcRtcSetTime (
   RegisterB.Bits.Set  = 1;
   RtcWrite (RTC_ADDRESS_REGISTER_B, RegisterB.Data);
 
-  ConvertEfiTimeToRtcTime (&RtcTime, RegisterB, &Century);
+  //
+  // Store the century value to RTC before converting to BCD format.
+  //
+  if (Global->CenturyRtcAddress != 0) {
+    RtcWrite (Global->CenturyRtcAddress, DecimalToBcd8 ((UINT8) (RtcTime.Year / 100)));
+  }
+
+  ConvertEfiTimeToRtcTime (&RtcTime, RegisterB);
 
   RtcWrite (RTC_ADDRESS_SECONDS, RtcTime.Second);
   RtcWrite (RTC_ADDRESS_MINUTES, RtcTime.Minute);
@@ -514,7 +633,6 @@ PcRtcSetTime (
   RtcWrite (RTC_ADDRESS_DAY_OF_THE_MONTH, RtcTime.Day);
   RtcWrite (RTC_ADDRESS_MONTH, RtcTime.Month);
   RtcWrite (RTC_ADDRESS_YEAR, (UINT8) RtcTime.Year);
-  RtcWrite (RTC_ADDRESS_CENTURY, Century);
 
   //
   // Allow updates of the RTC registers
@@ -564,7 +682,6 @@ PcRtcGetWakeupTime (
   EFI_STATUS      Status;
   RTC_REGISTER_B  RegisterB;
   RTC_REGISTER_C  RegisterC;
-  UINT8           Century;
   EFI_TIME        RtcTime;
   UINTN           DataSize;
 
@@ -612,8 +729,6 @@ PcRtcGetWakeupTime (
   Time->TimeZone = Global->SavedTimeZone;
   Time->Daylight = Global->Daylight;
 
-  Century = RtcRead (RTC_ADDRESS_CENTURY);
-
   //
   // Get the alarm info from variable
   //
@@ -644,7 +759,7 @@ PcRtcGetWakeupTime (
   //
   // Make sure all field values are in correct range
   //
-  Status = ConvertRtcTimeToEfiTime (Time, Century, RegisterB);
+  Status = ConvertRtcTimeToEfiTime (Time, RegisterB);
   if (!EFI_ERROR (Status)) {
     Status = RtcTimeFieldsValid (Time);
   }
@@ -680,7 +795,6 @@ PcRtcSetWakeupTime (
   EFI_STATUS            Status;
   EFI_TIME              RtcTime;
   RTC_REGISTER_B        RegisterB;
-  UINT8                 Century;
   EFI_TIME_CAPABILITIES Capabilities;
 
   ZeroMem (&RtcTime, sizeof (RtcTime));
@@ -736,7 +850,7 @@ PcRtcSetWakeupTime (
   RegisterB.Data = RtcRead (RTC_ADDRESS_REGISTER_B);
 
   if (Enable) {
-    ConvertEfiTimeToRtcTime (&RtcTime, RegisterB, &Century);
+    ConvertEfiTimeToRtcTime (&RtcTime, RegisterB);
   } else {
     //
     // if the alarm is disable, record the current setting.
@@ -837,7 +951,6 @@ CheckAndConvertBcd8ToDecimal8 (
 
   @param   Time       On input, the time data read from RTC to convert
                       On output, the time converted to UEFI format
-  @param   Century    Value of century read from RTC.
   @param   RegisterB  Value of Register B of RTC, indicating data mode
                       and hour format.
 
@@ -848,11 +961,11 @@ CheckAndConvertBcd8ToDecimal8 (
 EFI_STATUS
 ConvertRtcTimeToEfiTime (
   IN OUT EFI_TIME        *Time,
-  IN     UINT8           Century,
   IN     RTC_REGISTER_B  RegisterB
   )
 {
   BOOLEAN IsPM;
+  UINT8   Century;
 
   if ((Time->Hour & 0x80) != 0) {
     IsPM = TRUE;
@@ -870,14 +983,21 @@ ConvertRtcTimeToEfiTime (
     Time->Minute  = CheckAndConvertBcd8ToDecimal8 (Time->Minute);
     Time->Second  = CheckAndConvertBcd8ToDecimal8 (Time->Second);
   }
-  Century       = CheckAndConvertBcd8ToDecimal8 (Century);
 
   if (Time->Year == 0xff || Time->Month == 0xff || Time->Day == 0xff ||
-      Time->Hour == 0xff || Time->Minute == 0xff || Time->Second == 0xff ||
-      Century == 0xff) {
+      Time->Hour == 0xff || Time->Minute == 0xff || Time->Second == 0xff) {
     return EFI_INVALID_PARAMETER;
   }
 
+  //
+  // For minimal/maximum year range [1970, 2069],
+  //   Century is 19 if RTC year >= 70,
+  //   Century is 20 otherwise.
+  //
+  Century = (UINT8) (PcdGet16 (PcdMinimalValidYear) / 100);
+  if (Time->Year < PcdGet16 (PcdMinimalValidYear) % 100) {
+    Century++;
+  }
   Time->Year = (UINT16) (Century * 100 + Time->Year);
 
   //
@@ -985,28 +1105,13 @@ DayValid (
   IN  EFI_TIME  *Time
   )
 {
-  INTN  DayOfMonth[12];
-
-  DayOfMonth[0] = 31;
-  DayOfMonth[1] = 29;
-  DayOfMonth[2] = 31;
-  DayOfMonth[3] = 30;
-  DayOfMonth[4] = 31;
-  DayOfMonth[5] = 30;
-  DayOfMonth[6] = 31;
-  DayOfMonth[7] = 31;
-  DayOfMonth[8] = 30;
-  DayOfMonth[9] = 31;
-  DayOfMonth[10] = 30;
-  DayOfMonth[11] = 31;
-
   //
   // The validity of Time->Month field should be checked before
   //
   ASSERT (Time->Month >=1);
   ASSERT (Time->Month <=12);
   if (Time->Day < 1 ||
-      Time->Day > DayOfMonth[Time->Month - 1] ||
+      Time->Day > mDayOfMonth[Time->Month - 1] ||
       (Time->Month == 2 && (!IsLeapYear (Time) && Time->Day > 28))
       ) {
     return FALSE;
@@ -1044,23 +1149,20 @@ IsLeapYear (
 }
 
 /**
-  Converts time from EFI_TIME format defined by UEFI spec to RTC's.
+  Converts time from EFI_TIME format defined by UEFI spec to RTC format.
 
-  This function converts time from EFI_TIME format defined by UEFI spec to RTC's.
+  This function converts time from EFI_TIME format defined by UEFI spec to RTC format.
   If data mode of RTC is BCD, then converts EFI_TIME to it.
   If RTC is in 12-hour format, then converts EFI_TIME to it.
 
   @param   Time       On input, the time data read from UEFI to convert
                       On output, the time converted to RTC format
   @param   RegisterB  Value of Register B of RTC, indicating data mode
-  @param   Century    It is set according to EFI_TIME Time.
-
 **/
 VOID
 ConvertEfiTimeToRtcTime (
   IN OUT EFI_TIME        *Time,
-  IN     RTC_REGISTER_B  RegisterB,
-     OUT UINT8           *Century
+  IN     RTC_REGISTER_B  RegisterB
   )
 {
   BOOLEAN IsPM;
@@ -1081,10 +1183,8 @@ ConvertEfiTimeToRtcTime (
     }
   }
   //
-  // Set the Time/Date/Daylight Savings values.
+  // Set the Time/Date values.
   //
-  *Century    = DecimalToBcd8 ((UINT8) (Time->Year / 100));
-
   Time->Year  = (UINT16) (Time->Year % 100);
 
   if (RegisterB.Bits.Dm == 0) {
@@ -1147,22 +1247,8 @@ IsWithinOneDay (
   IN EFI_TIME  *To
   )
 {
-  UINT8   DayOfMonth[12];
   BOOLEAN Adjacent;
 
-  DayOfMonth[0] = 31;
-  DayOfMonth[1] = 29;
-  DayOfMonth[2] = 31;
-  DayOfMonth[3] = 30;
-  DayOfMonth[4] = 31;
-  DayOfMonth[5] = 30;
-  DayOfMonth[6] = 31;
-  DayOfMonth[7] = 31;
-  DayOfMonth[8] = 30;
-  DayOfMonth[9] = 31;
-  DayOfMonth[10] = 30;
-  DayOfMonth[11] = 31;
-
   Adjacent = FALSE;
 
   //
@@ -1189,7 +1275,7 @@ IsWithinOneDay (
             Adjacent = TRUE;
           }
         }
-      } else if (From->Day == DayOfMonth[From->Month - 1]) {
+      } else if (From->Day == mDayOfMonth[From->Month - 1]) {
         if ((CompareHMS(From, To) >= 0)) {
            Adjacent = TRUE;
         }
@@ -1208,3 +1294,62 @@ IsWithinOneDay (
   return Adjacent;
 }
 
+/**
+  Get the century RTC address from the ACPI FADT table.
+
+  @return  The century RTC address or 0 if not found.
+**/
+UINT8
+GetCenturyRtcAddress (
+  VOID
+  )
+{
+  EFI_ACPI_2_0_FIXED_ACPI_DESCRIPTION_TABLE     *Fadt;
+
+  Fadt = (EFI_ACPI_2_0_FIXED_ACPI_DESCRIPTION_TABLE *) EfiLocateFirstAcpiTable (
+                                                         EFI_ACPI_2_0_FIXED_ACPI_DESCRIPTION_TABLE_SIGNATURE
+                                                         );
+
+  if ((Fadt != NULL) &&
+      (Fadt->Century > RTC_ADDRESS_REGISTER_D) && (Fadt->Century < 0x80)
+      ) {
+    return Fadt->Century;
+  } else {
+    return 0;
+  }
+}
+
+/**
+  Notification function of ACPI Table change.
+
+  This is a notification function registered on ACPI Table change event.
+  It saves the Century address stored in ACPI FADT table.
+
+  @param  Event        Event whose notification function is being invoked.
+  @param  Context      Pointer to the notification function's context.
+
+**/
+VOID
+EFIAPI
+PcRtcAcpiTableChangeCallback (
+  IN EFI_EVENT        Event,
+  IN VOID             *Context
+  )
+{
+  EFI_STATUS          Status;
+  EFI_TIME            Time;
+  UINT8               CenturyRtcAddress;
+  UINT8               Century;
+
+  CenturyRtcAddress = GetCenturyRtcAddress ();
+  if ((CenturyRtcAddress != 0) && (mModuleGlobal.CenturyRtcAddress != CenturyRtcAddress)) {
+    mModuleGlobal.CenturyRtcAddress = CenturyRtcAddress;
+    Status = PcRtcGetTime (&Time, NULL, &mModuleGlobal);
+    if (!EFI_ERROR (Status)) {
+      Century = (UINT8) (Time.Year / 100);
+      Century = DecimalToBcd8 (Century);
+      DEBUG ((EFI_D_INFO, "PcRtc: Write 0x%x to CMOS location 0x%x\n", Century, mModuleGlobal.CenturyRtcAddress));
+      RtcWrite (mModuleGlobal.CenturyRtcAddress, Century);
+    }
+  }
+}
