@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -1062,6 +1072,31 @@ static DECLCALLBACK(bool) drvdiskintIsNonRotational(PPDMIMEDIA pInterface)
     return pThis->pDrvMedia->pfnIsNonRotational(pThis->pDrvMedia);
 }
 
+/** @interface_method_impl{PDMIMEDIA,pfnGetRegionCount} */
+static DECLCALLBACK(uint32_t) drvdiskintGetRegionCount(PPDMIMEDIA pInterface)
+{
+    PDRVDISKINTEGRITY pThis = PDMIMEDIA_2_DRVDISKINTEGRITY(pInterface);
+    return pThis->pDrvMedia->pfnGetRegionCount(pThis->pDrvMedia);
+}
+
+/** @interface_method_impl{PDMIMEDIA,pfnQueryRegionProperties} */
+static DECLCALLBACK(int) drvdiskintQueryRegionProperties(PPDMIMEDIA pInterface, uint32_t uRegion, uint64_t *pu64LbaStart,
+                                                         uint64_t *pcBlocks, uint64_t *pcbBlock,
+                                                         PVDREGIONDATAFORM penmDataForm)
+{
+    PDRVDISKINTEGRITY pThis = PDMIMEDIA_2_DRVDISKINTEGRITY(pInterface);
+    return pThis->pDrvMedia->pfnQueryRegionProperties(pThis->pDrvMedia, uRegion, pu64LbaStart, pcBlocks, pcbBlock, penmDataForm);
+}
+
+/** @interface_method_impl{PDMIMEDIA,pfnQueryRegionPropertiesForLba} */
+static DECLCALLBACK(int) drvdiskintQueryRegionPropertiesForLba(PPDMIMEDIA pInterface, uint64_t u64LbaStart,
+                                                               uint32_t *puRegion, uint64_t *pcBlocks,
+                                                               uint64_t *pcbBlock, PVDREGIONDATAFORM penmDataForm)
+{
+    PDRVDISKINTEGRITY pThis = PDMIMEDIA_2_DRVDISKINTEGRITY(pInterface);
+    return pThis->pDrvMedia->pfnQueryRegionPropertiesForLba(pThis->pDrvMedia, u64LbaStart, puRegion, pcBlocks, pcbBlock, penmDataForm);
+}
+
 /* -=-=-=-=- IMediaPort -=-=-=-=- */
 
 /** Makes a PDRVBLOCK out of a PPDMIMEDIAPORT. */
@@ -1310,6 +1345,15 @@ static DECLCALLBACK(int) drvdiskintQueryFeatures(PPDMIMEDIAEX pInterface, uint32
     return pThis->pDrvMediaEx->pfnQueryFeatures(pThis->pDrvMediaEx, pfFeatures);
 }
 
+/**
+ * @interface_method_impl{PDMIMEDIAEX,pfnNotifySuspend}
+ */
+static DECLCALLBACK(void) drvdiskintNotifySuspend(PPDMIMEDIAEX pInterface)
+{
+    PDRVDISKINTEGRITY pThis = RT_FROM_MEMBER(pInterface, DRVDISKINTEGRITY, IMediaEx);
+    return pThis->pDrvMediaEx->pfnNotifySuspend(pThis->pDrvMediaEx);
+}
+
 /**
  * @interface_method_impl{PDMIMEDIAEX,pfnIoReqAllocSizeSet}
  */
@@ -1772,80 +1816,81 @@ static DECLCALLBACK(void) drvdiskintDestruct(PPDMDRVINS pDrvIns)
  */
 static DECLCALLBACK(int) drvdiskintConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags)
 {
-    int rc = VINF_SUCCESS;
-    PDRVDISKINTEGRITY pThis = PDMINS_2_DATA(pDrvIns, PDRVDISKINTEGRITY);
-    LogFlow(("drvdiskintConstruct: iInstance=%d\n", pDrvIns->iInstance));
     PDMDRV_CHECK_VERSIONS_RETURN(pDrvIns);
+    PDRVDISKINTEGRITY   pThis = PDMINS_2_DATA(pDrvIns, PDRVDISKINTEGRITY);
+    PCPDMDRVHLPR3       pHlp  = pDrvIns->pHlpR3;
+
+    LogFlow(("drvdiskintConstruct: iInstance=%d\n", pDrvIns->iInstance));
 
     /*
      * Validate configuration.
      */
-    if (!CFGMR3AreValuesValid(pCfg, "CheckConsistency\0"
-                                    "TraceRequests\0"
-                                    "CheckIntervalMs\0"
-                                    "ExpireIntervalMs\0"
-                                    "CheckDoubleCompletions\0"
-                                    "HistorySize\0"
-                                    "IoLogType\0"
-                                    "IoLogFile\0"
-                                    "IoLogAddress\0"
-                                    "IoLogPort\0"
-                                    "IoLogData\0"
-                                    "PrepopulateRamDisk\0"
-                                    "ReadAfterWrite\0"
-                                    "RecordWriteBeforeCompletion\0"
-                                    "ValidateMemoryBuffers\0"))
-        return VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES;
+    PDMDRV_VALIDATE_CONFIG_RETURN(pDrvIns,  "CheckConsistency"
+                                            "|TraceRequests"
+                                            "|CheckIntervalMs"
+                                            "|ExpireIntervalMs"
+                                            "|CheckDoubleCompletions"
+                                            "|HistorySize"
+                                            "|IoLogType"
+                                            "|IoLogFile"
+                                            "|IoLogAddress"
+                                            "|IoLogPort"
+                                            "|IoLogData"
+                                            "|PrepopulateRamDisk"
+                                            "|ReadAfterWrite"
+                                            "|RecordWriteBeforeCompletion"
+                                            "|ValidateMemoryBuffers",
+                                            "");
 
-    rc = CFGMR3QueryBoolDef(pCfg, "CheckConsistency", &pThis->fCheckConsistency, false);
+    int rc = pHlp->pfnCFGMQueryBoolDef(pCfg, "CheckConsistency", &pThis->fCheckConsistency, false);
     AssertRC(rc);
-    rc = CFGMR3QueryBoolDef(pCfg, "TraceRequests", &pThis->fTraceRequests, false);
+    rc = pHlp->pfnCFGMQueryBoolDef(pCfg, "TraceRequests", &pThis->fTraceRequests, false);
     AssertRC(rc);
-    rc = CFGMR3QueryU32Def(pCfg, "CheckIntervalMs", &pThis->uCheckIntervalMs, 5000); /* 5 seconds */
+    rc = pHlp->pfnCFGMQueryU32Def(pCfg, "CheckIntervalMs", &pThis->uCheckIntervalMs, 5000); /* 5 seconds */
     AssertRC(rc);
-    rc = CFGMR3QueryU32Def(pCfg, "ExpireIntervalMs", &pThis->uExpireIntervalMs, 20000); /* 20 seconds */
+    rc = pHlp->pfnCFGMQueryU32Def(pCfg, "ExpireIntervalMs", &pThis->uExpireIntervalMs, 20000); /* 20 seconds */
     AssertRC(rc);
-    rc = CFGMR3QueryBoolDef(pCfg, "CheckDoubleCompletions", &pThis->fCheckDoubleCompletion, false);
+    rc = pHlp->pfnCFGMQueryBoolDef(pCfg, "CheckDoubleCompletions", &pThis->fCheckDoubleCompletion, false);
     AssertRC(rc);
-    rc = CFGMR3QueryU32Def(pCfg, "HistorySize", &pThis->cEntries, 512);
+    rc = pHlp->pfnCFGMQueryU32Def(pCfg, "HistorySize", &pThis->cEntries, 512);
     AssertRC(rc);
-    rc = CFGMR3QueryBoolDef(pCfg, "PrepopulateRamDisk", &pThis->fPrepopulateRamDisk, false);
+    rc = pHlp->pfnCFGMQueryBoolDef(pCfg, "PrepopulateRamDisk", &pThis->fPrepopulateRamDisk, false);
     AssertRC(rc);
-    rc = CFGMR3QueryBoolDef(pCfg, "ReadAfterWrite", &pThis->fReadAfterWrite, false);
+    rc = pHlp->pfnCFGMQueryBoolDef(pCfg, "ReadAfterWrite", &pThis->fReadAfterWrite, false);
     AssertRC(rc);
-    rc = CFGMR3QueryBoolDef(pCfg, "RecordWriteBeforeCompletion", &pThis->fRecordWriteBeforeCompletion, false);
+    rc = pHlp->pfnCFGMQueryBoolDef(pCfg, "RecordWriteBeforeCompletion", &pThis->fRecordWriteBeforeCompletion, false);
     AssertRC(rc);
-    rc = CFGMR3QueryBoolDef(pCfg, "ValidateMemoryBuffers", &pThis->fValidateMemBufs, false);
+    rc = pHlp->pfnCFGMQueryBoolDef(pCfg, "ValidateMemoryBuffers", &pThis->fValidateMemBufs, false);
     AssertRC(rc);
 
     bool fIoLogData = false;
-    rc = CFGMR3QueryBoolDef(pCfg, "IoLogData", &fIoLogData, false);
+    rc = pHlp->pfnCFGMQueryBoolDef(pCfg, "IoLogData", &fIoLogData, false);
     AssertRC(rc);
 
     char *pszIoLogType = NULL;
     char *pszIoLogFilename = NULL;
     char *pszAddress = NULL;
     uint32_t uPort = 0;
-    rc = CFGMR3QueryStringAlloc(pCfg, "IoLogType", &pszIoLogType);
+    rc = pHlp->pfnCFGMQueryStringAlloc(pCfg, "IoLogType", &pszIoLogType);
     if (RT_SUCCESS(rc))
     {
         if (!RTStrICmp(pszIoLogType, "File"))
         {
-            rc = CFGMR3QueryStringAlloc(pCfg, "IoLogFile", &pszIoLogFilename);
+            rc = pHlp->pfnCFGMQueryStringAlloc(pCfg, "IoLogFile", &pszIoLogFilename);
             AssertRC(rc);
         }
         else if (!RTStrICmp(pszIoLogType, "Server"))
         {
-            rc = CFGMR3QueryStringAllocDef(pCfg, "IoLogAddress", &pszAddress, NULL);
+            rc = pHlp->pfnCFGMQueryStringAllocDef(pCfg, "IoLogAddress", &pszAddress, NULL);
             AssertRC(rc);
-            rc = CFGMR3QueryU32Def(pCfg, "IoLogPort", &uPort, 4000);
+            rc = pHlp->pfnCFGMQueryU32Def(pCfg, "IoLogPort", &uPort, 4000);
             AssertRC(rc);
         }
         else if (!RTStrICmp(pszIoLogType, "Client"))
         {
-            rc = CFGMR3QueryStringAlloc(pCfg, "IoLogAddress", &pszAddress);
+            rc = pHlp->pfnCFGMQueryStringAlloc(pCfg, "IoLogAddress", &pszAddress);
             AssertRC(rc);
-            rc = CFGMR3QueryU32Def(pCfg, "IoLogPort", &uPort, 4000);
+            rc = pHlp->pfnCFGMQueryU32Def(pCfg, "IoLogPort", &uPort, 4000);
             AssertRC(rc);
         }
         else
@@ -1864,24 +1909,30 @@ static DECLCALLBACK(int) drvdiskintConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg,
     pDrvIns->IBase.pfnQueryInterface     = drvdiskintQueryInterface;
 
     /* IMedia */
-    pThis->IMedia.pfnRead                = drvdiskintRead;
-    pThis->IMedia.pfnWrite               = drvdiskintWrite;
-    pThis->IMedia.pfnFlush               = drvdiskintFlush;
-    pThis->IMedia.pfnGetSize             = drvdiskintGetSize;
-    pThis->IMedia.pfnIsReadOnly          = drvdiskintIsReadOnly;
-    pThis->IMedia.pfnBiosIsVisible       = drvdiskintBiosIsVisible;
-    pThis->IMedia.pfnBiosGetPCHSGeometry = drvdiskintBiosGetPCHSGeometry;
-    pThis->IMedia.pfnBiosSetPCHSGeometry = drvdiskintBiosSetPCHSGeometry;
-    pThis->IMedia.pfnBiosGetLCHSGeometry = drvdiskintBiosGetLCHSGeometry;
-    pThis->IMedia.pfnBiosSetLCHSGeometry = drvdiskintBiosSetLCHSGeometry;
-    pThis->IMedia.pfnGetUuid             = drvdiskintGetUuid;
-    pThis->IMedia.pfnGetSectorSize       = drvdiskintGetSectorSize;
-    pThis->IMedia.pfnGetType             = drvdiskintGetType;
-    pThis->IMedia.pfnReadPcBios          = drvdiskintReadPcBios;
-    pThis->IMedia.pfnIsNonRotational     = drvdiskintIsNonRotational;
+    pThis->IMedia.pfnRead                        = drvdiskintRead;
+    pThis->IMedia.pfnWrite                       = drvdiskintWrite;
+    pThis->IMedia.pfnFlush                       = drvdiskintFlush;
+    pThis->IMedia.pfnGetSize                     = drvdiskintGetSize;
+    pThis->IMedia.pfnIsReadOnly                  = drvdiskintIsReadOnly;
+    pThis->IMedia.pfnBiosIsVisible               = drvdiskintBiosIsVisible;
+    pThis->IMedia.pfnBiosGetPCHSGeometry         = drvdiskintBiosGetPCHSGeometry;
+    pThis->IMedia.pfnBiosSetPCHSGeometry         = drvdiskintBiosSetPCHSGeometry;
+    pThis->IMedia.pfnBiosGetLCHSGeometry         = drvdiskintBiosGetLCHSGeometry;
+    pThis->IMedia.pfnBiosSetLCHSGeometry         = drvdiskintBiosSetLCHSGeometry;
+    pThis->IMedia.pfnGetUuid                     = drvdiskintGetUuid;
+    pThis->IMedia.pfnGetSectorSize               = drvdiskintGetSectorSize;
+    pThis->IMedia.pfnGetType                     = drvdiskintGetType;
+    pThis->IMedia.pfnReadPcBios                  = drvdiskintReadPcBios;
+    pThis->IMedia.pfnIsNonRotational             = drvdiskintIsNonRotational;
+    pThis->IMedia.pfnSendCmd                     = NULL;
+    pThis->IMedia.pfnGetRegionCount              = drvdiskintGetRegionCount;
+    pThis->IMedia.pfnQueryRegionProperties       = drvdiskintQueryRegionProperties;
+    pThis->IMedia.pfnQueryRegionPropertiesForLba = drvdiskintQueryRegionPropertiesForLba;
+
 
     /* IMediaEx. */
     pThis->IMediaEx.pfnQueryFeatures            = drvdiskintQueryFeatures;
+    pThis->IMediaEx.pfnNotifySuspend            = drvdiskintNotifySuspend;
     pThis->IMediaEx.pfnIoReqAllocSizeSet        = drvdiskintIoReqAllocSizeSet;
     pThis->IMediaEx.pfnIoReqAlloc               = drvdiskintIoReqAlloc;
     pThis->IMediaEx.pfnIoReqFree                = drvdiskintIoReqFree;
@@ -1995,23 +2046,23 @@ static DECLCALLBACK(int) drvdiskintConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg,
         if (!RTStrICmp(pszIoLogType, "File"))
         {
             rc = RTTraceLogWrCreateFile(&pThis->hIoLogger, NULL, pszIoLogFilename);
-            MMR3HeapFree(pszIoLogFilename);
+            PDMDrvHlpMMHeapFree(pDrvIns, pszIoLogFilename);
         }
         else if (!RTStrICmp(pszIoLogType, "Server"))
         {
             rc = RTTraceLogWrCreateTcpServer(&pThis->hIoLogger, NULL, pszAddress, uPort);
             if (pszAddress)
-                MMR3HeapFree(pszAddress);
+                PDMDrvHlpMMHeapFree(pDrvIns, pszAddress);
         }
         else if (!RTStrICmp(pszIoLogType, "Client"))
         {
             rc = RTTraceLogWrCreateTcpClient(&pThis->hIoLogger, NULL, pszAddress, uPort);
-            MMR3HeapFree(pszAddress);
+            PDMDrvHlpMMHeapFree(pDrvIns, pszAddress);
         }
         else
             AssertMsgFailed(("Invalid I/O log type given: %s\n", pszIoLogType));
 
-        MMR3HeapFree(pszIoLogType);
+        PDMDrvHlpMMHeapFree(pDrvIns, pszIoLogType);
     }
 
     /* Read in all data before the start if requested. */
