@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -58,7 +68,7 @@ static void *GetStdDescSync(PUSBPROXYDEV pProxyDev, uint8_t iDescType, uint8_t i
     int cRetries = 0;
     uint16_t cbInitialHint = cbHint;
 
-    LogFlow(("GetStdDescSync: pProxyDev=%s\n", pProxyDev->pUsbIns->pszName));
+    LogFlow(("GetStdDescSync: pProxyDev=%s, iDescType=%d, iIdx=%d, LangId=%04X, cbHint=%u\n", pProxyDev->pUsbIns->pszName, iDescType, iIdx, LangId, cbHint));
     for (;;)
     {
         /*
@@ -67,20 +77,16 @@ static void *GetStdDescSync(PUSBPROXYDEV pProxyDev, uint8_t iDescType, uint8_t i
         int rc = VINF_SUCCESS;
         VUSBURB Urb;
         AssertCompile(RT_SIZEOFMEMB(VUSBURB, abData) >= _4K);
+        RT_ZERO(Urb);
         Urb.u32Magic      = VUSBURB_MAGIC;
         Urb.enmState      = VUSBURBSTATE_IN_FLIGHT;
         Urb.pszDesc       = (char*)"URB sync";
-        Urb.pHci          = NULL;
-        Urb.paTds         = NULL;
-        Urb.Dev.pvPrivate = NULL;
-        Urb.Dev.pNext     = NULL;
         Urb.DstAddress    = 0;
         Urb.EndPt         = 0;
         Urb.enmType       = VUSBXFERTYPE_MSG;
         Urb.enmDir        = VUSBDIRECTION_IN;
         Urb.fShortNotOk   = false;
         Urb.enmStatus     = VUSBSTATUS_INVALID;
-        Urb.pVUsb         = NULL;
         cbHint = RT_MIN(cbHint, sizeof(Urb.abData) - sizeof(VUSBSETUP));
         Urb.cbData = cbHint + sizeof(VUSBSETUP);
 
@@ -98,7 +104,7 @@ static void *GetStdDescSync(PUSBPROXYDEV pProxyDev, uint8_t iDescType, uint8_t i
         rc = pProxyDev->pOps->pfnUrbQueue(pProxyDev, &Urb);
         if (RT_FAILURE(rc))
         {
-            Log(("GetStdDescSync: pfnUrbReap failed, rc=%d\n", rc));
+            Log(("GetStdDescSync: pfnUrbQueue failed, rc=%d\n", rc));
             goto err;
         }
 
@@ -109,6 +115,7 @@ static void *GetStdDescSync(PUSBPROXYDEV pProxyDev, uint8_t iDescType, uint8_t i
         pUrbReaped = pProxyDev->pOps->pfnUrbReap(pProxyDev, 5000 /* ms */);
         if (!pUrbReaped)
         {
+            Log(("GetStdDescSync: pfnUrbReap returned NULL, cancel and re-reap\n"));
             rc = pProxyDev->pOps->pfnUrbCancel(pProxyDev, &Urb);
             AssertRC(rc);
             /** @todo This breaks the comment above... */
@@ -157,7 +164,11 @@ static void *GetStdDescSync(PUSBPROXYDEV pProxyDev, uint8_t iDescType, uint8_t i
             Log(("GetStdDescSync: Part descriptor, Urb.cbData=%u, cbDesc=%u cbHint=%u\n", Urb.cbData, cbDesc, cbHint));
 
             if (cbHint > sizeof(Urb.abData))
-                Log(("GetStdDescSync: cbHint=%u, Urb.abData=%u\n", cbHint, sizeof(Urb.abData)));
+            {
+                Log(("GetStdDescSync: cbHint=%u, Urb.abData=%u, retrying immediately\n", cbHint, sizeof(Urb.abData)));
+                /* Not an error, go again without incrementing retry count or delaying. */
+                continue;
+            }
 
             goto err;
         }
@@ -814,28 +825,31 @@ static DECLCALLBACK(void) usbProxyDestruct(PPDMUSBINS pUsbIns)
  * @returns VBox status code.
  * @param   pFilter         The filter.
  * @param   enmFieldIdx     The filter field indext.
+ * @param   pHlp            The USB helper callback table.
  * @param   pNode           The CFGM node.
  * @param   pszExact        The exact value name.
  * @param   pszExpr         The expression value name.
  */
-static int usbProxyQueryNum(PUSBFILTER pFilter, USBFILTERIDX enmFieldIdx, PCFGMNODE pNode, const char *pszExact, const char *pszExpr)
+static int usbProxyQueryNum(PUSBFILTER pFilter, USBFILTERIDX enmFieldIdx,
+                            PCPDMUSBHLP pHlp, PCFGMNODE pNode,
+                            const char *pszExact, const char *pszExpr)
 {
     char szTmp[256];
 
     /* try exact first */
     uint16_t u16;
-    int rc = CFGMR3QueryU16(pNode, pszExact, &u16);
+    int rc = pHlp->pfnCFGMQueryU16(pNode, pszExact, &u16);
     if (RT_SUCCESS(rc))
     {
         rc = USBFilterSetNumExact(pFilter, enmFieldIdx, u16, true);
         AssertRCReturn(rc, rc);
 
         /* make sure only the exact attribute is present. */
-        rc = CFGMR3QueryString(pNode, pszExpr, szTmp, sizeof(szTmp));
+        rc = pHlp->pfnCFGMQueryString(pNode, pszExpr, szTmp, sizeof(szTmp));
         if (RT_UNLIKELY(rc != VERR_CFGM_VALUE_NOT_FOUND))
         {
             szTmp[0] = '\0';
-            CFGMR3GetName(pNode, szTmp, sizeof(szTmp));
+            pHlp->pfnCFGMGetName(pNode, szTmp, sizeof(szTmp));
             LogRel(("usbProxyConstruct: %s: Both %s and %s are present!\n", szTmp, pszExact, pszExpr));
             return VERR_INVALID_PARAMETER;
         }
@@ -844,13 +858,13 @@ static int usbProxyQueryNum(PUSBFILTER pFilter, USBFILTERIDX enmFieldIdx, PCFGMN
     if (RT_UNLIKELY(rc != VERR_CFGM_VALUE_NOT_FOUND))
     {
         szTmp[0] = '\0';
-        CFGMR3GetName(pNode, szTmp, sizeof(szTmp));
+        pHlp->pfnCFGMGetName(pNode, szTmp, sizeof(szTmp));
         LogRel(("usbProxyConstruct: %s: %s query failed, rc=%Rrc\n", szTmp, pszExact, rc));
         return rc;
     }
 
     /* expression? */
-    rc = CFGMR3QueryString(pNode, pszExpr, szTmp, sizeof(szTmp));
+    rc = pHlp->pfnCFGMQueryString(pNode, pszExpr, szTmp, sizeof(szTmp));
     if (RT_SUCCESS(rc))
     {
         rc = USBFilterSetNumExpression(pFilter, enmFieldIdx, szTmp, true);
@@ -860,7 +874,7 @@ static int usbProxyQueryNum(PUSBFILTER pFilter, USBFILTERIDX enmFieldIdx, PCFGMN
     if (RT_UNLIKELY(rc != VERR_CFGM_VALUE_NOT_FOUND))
     {
         szTmp[0] = '\0';
-        CFGMR3GetName(pNode, szTmp, sizeof(szTmp));
+        pHlp->pfnCFGMGetName(pNode, szTmp, sizeof(szTmp));
         LogRel(("usbProxyConstruct: %s: %s query failed, rc=%Rrc\n", szTmp, pszExpr, rc));
         return rc;
     }
@@ -874,7 +888,9 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
 {
     PDMUSB_CHECK_VERSIONS_RETURN(pUsbIns);
     RT_NOREF(iInstance);
-    PUSBPROXYDEV pThis = PDMINS_2_DATA(pUsbIns, PUSBPROXYDEV);
+    PUSBPROXYDEV    pThis = PDMINS_2_DATA(pUsbIns, PUSBPROXYDEV);
+    PCPDMUSBHLP     pHlp  = pUsbIns->pHlpR3;
+
     LogFlow(("usbProxyConstruct: pUsbIns=%p iInstance=%d\n", pUsbIns, iInstance));
 
     /*
@@ -891,15 +907,11 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
      * Read the basic configuration.
      */
     char szAddress[1024];
-    int rc = CFGMR3QueryString(pCfg, "Address", szAddress, sizeof(szAddress));
+    int rc = pHlp->pfnCFGMQueryString(pCfg, "Address", szAddress, sizeof(szAddress));
     AssertRCReturn(rc, rc);
 
     char szBackend[64];
-    rc = CFGMR3QueryString(pCfg, "Backend", szBackend, sizeof(szBackend));
-    AssertRCReturn(rc, rc);
-
-    void *pvBackend;
-    rc = CFGMR3QueryPtr(pCfg, "pvBackend", &pvBackend);
+    rc = pHlp->pfnCFGMQueryString(pCfg, "Backend", szBackend, sizeof(szBackend));
     AssertRCReturn(rc, rc);
 
     /*
@@ -922,7 +934,7 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
     if (!pThis->pvInstanceDataR3)
         return PDMUSB_SET_ERROR(pUsbIns, VERR_NO_MEMORY, N_("USBProxy: can't allocate memory for host backend"));
 
-    rc = pThis->pOps->pfnOpen(pThis, szAddress, pvBackend);
+    rc = pThis->pOps->pfnOpen(pThis, szAddress);
     if (RT_FAILURE(rc))
     {
         LogRel(("usbProxyConstruct: Failed to open '%s', rc=%Rrc\n", szAddress, rc));
@@ -975,7 +987,7 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
      * be useful).
      */
     PCFGMNODE pCfgGlobalDev = pCfgGlobal;
-    PCFGMNODE pCur = CFGMR3GetFirstChild(pCfgGlobal);
+    PCFGMNODE pCur = pHlp->pfnCFGMGetFirstChild(pCfgGlobal);
     if (pCur)
     {
         /*
@@ -994,7 +1006,7 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
 
         int iBestMatchRate = -1;
         PCFGMNODE pBestMatch = NULL;
-        for (pCur = CFGMR3GetFirstChild(pCfgGlobal); pCur; pCur = CFGMR3GetNextChild(pCur))
+        for (pCur = pHlp->pfnCFGMGetFirstChild(pCfgGlobal); pCur; pCur = pHlp->pfnCFGMGetNextChild(pCur))
         {
             /*
              * Construct a filter from the attributes in the node.
@@ -1003,25 +1015,25 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
             USBFilterInit(&Filter, USBFILTERTYPE_CAPTURE);
 
             /* numeric */
-            if (    RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_VENDOR_ID,        pCur, "idVendor",        "idVendorExpr"))
-                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_PRODUCT_ID,       pCur, "idProduct",       "idProcutExpr"))
-                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_DEVICE_REV,       pCur, "bcdDevice",       "bcdDeviceExpr"))
-                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_DEVICE_CLASS,     pCur, "bDeviceClass",    "bDeviceClassExpr"))
-                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_DEVICE_SUB_CLASS, pCur, "bDeviceSubClass", "bDeviceSubClassExpr"))
-                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_DEVICE_PROTOCOL,  pCur, "bDeviceProtocol", "bDeviceProtocolExpr")))
+            if (    RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_VENDOR_ID,        pHlp, pCur, "idVendor",        "idVendorExpr"))
+                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_PRODUCT_ID,       pHlp, pCur, "idProduct",       "idProcutExpr"))
+                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_DEVICE_REV,       pHlp, pCur, "bcdDevice",       "bcdDeviceExpr"))
+                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_DEVICE_CLASS,     pHlp, pCur, "bDeviceClass",    "bDeviceClassExpr"))
+                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_DEVICE_SUB_CLASS, pHlp, pCur, "bDeviceSubClass", "bDeviceSubClassExpr"))
+                ||  RT_FAILURE(usbProxyQueryNum(&Filter, USBFILTERIDX_DEVICE_PROTOCOL,  pHlp, pCur, "bDeviceProtocol", "bDeviceProtocolExpr")))
                 continue; /* skip it */
 
             /* strings */
             /** @todo manufacturer, product and serial strings */
 
             /* ignore unknown config values, but not without bitching. */
-            if (!CFGMR3AreValuesValid(pCur,
-                                      "idVendor\0idVendorExpr\0"
-                                      "idProduct\0idProductExpr\0"
-                                      "bcdDevice\0bcdDeviceExpr\0"
-                                      "bDeviceClass\0bDeviceClassExpr\0"
-                                      "bDeviceSubClass\0bDeviceSubClassExpr\0"
-                                      "bDeviceProtocol\0bDeviceProtocolExpr\0"))
+            if (!pHlp->pfnCFGMAreValuesValid(pCur,
+                                             "idVendor\0idVendorExpr\0"
+                                             "idProduct\0idProductExpr\0"
+                                             "bcdDevice\0bcdDeviceExpr\0"
+                                             "bDeviceClass\0bDeviceClassExpr\0"
+                                             "bDeviceSubClass\0bDeviceSubClassExpr\0"
+                                             "bDeviceProtocol\0bDeviceProtocolExpr\0"))
                 LogRel(("usbProxyConstruct: Unknown value(s) in config filter (ignored)!\n"));
 
             /*
@@ -1038,7 +1050,7 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
             }
         }
         if (pBestMatch)
-            pCfgGlobalDev = CFGMR3GetChild(pBestMatch, "Config");
+            pCfgGlobalDev = pHlp->pfnCFGMGetChild(pBestMatch, "Config");
         if (pCfgGlobalDev)
             pCfgGlobalDev = pCfgGlobal;
     }
@@ -1046,45 +1058,45 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
     /*
      * Query the rest of the configuration using the global as fallback.
      */
-    rc = CFGMR3QueryU32(pCfg, "MaskedIfs", &pThis->fMaskedIfs);
+    rc = pHlp->pfnCFGMQueryU32(pCfg, "MaskedIfs", &pThis->fMaskedIfs);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
-        rc = CFGMR3QueryU32(pCfgGlobalDev, "MaskedIfs", &pThis->fMaskedIfs);
+        rc = pHlp->pfnCFGMQueryU32(pCfgGlobalDev, "MaskedIfs", &pThis->fMaskedIfs);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         pThis->fMaskedIfs = 0;
     else
         AssertRCReturn(rc, rc);
 
     bool fForce11Device;
-    rc = CFGMR3QueryBool(pCfg, "Force11Device", &fForce11Device);
+    rc = pHlp->pfnCFGMQueryBool(pCfg, "Force11Device", &fForce11Device);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
-        rc = CFGMR3QueryBool(pCfgGlobalDev, "Force11Device", &fForce11Device);
+        rc = pHlp->pfnCFGMQueryBool(pCfgGlobalDev, "Force11Device", &fForce11Device);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         fForce11Device = false;
     else
         AssertRCReturn(rc, rc);
 
     bool fForce11PacketSize;
-    rc = CFGMR3QueryBool(pCfg, "Force11PacketSize", &fForce11PacketSize);
+    rc = pHlp->pfnCFGMQueryBool(pCfg, "Force11PacketSize", &fForce11PacketSize);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
-        rc = CFGMR3QueryBool(pCfgGlobalDev, "Force11PacketSize", &fForce11PacketSize);
+        rc = pHlp->pfnCFGMQueryBool(pCfgGlobalDev, "Force11PacketSize", &fForce11PacketSize);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         fForce11PacketSize = false;
     else
         AssertRCReturn(rc, rc);
 
     bool fEditAudioSyncEp;
-    rc = CFGMR3QueryBool(pCfg, "EditAudioSyncEp", &fEditAudioSyncEp);
+    rc = pHlp->pfnCFGMQueryBool(pCfg, "EditAudioSyncEp", &fEditAudioSyncEp);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
-        rc = CFGMR3QueryBool(pCfgGlobalDev, "EditAudioSyncEp", &fEditAudioSyncEp);
+        rc = pHlp->pfnCFGMQueryBool(pCfgGlobalDev, "EditAudioSyncEp", &fEditAudioSyncEp);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         fEditAudioSyncEp = true;    /* NB: On by default! */
     else
         AssertRCReturn(rc, rc);
 
     bool fEditRemoteWake;
-    rc = CFGMR3QueryBool(pCfg, "EditRemoteWake", &fEditRemoteWake);
+    rc = pHlp->pfnCFGMQueryBool(pCfg, "EditRemoteWake", &fEditRemoteWake);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
-        rc = CFGMR3QueryBool(pCfgGlobalDev, "EditRemoteWake", &fEditRemoteWake);
+        rc = pHlp->pfnCFGMQueryBool(pCfgGlobalDev, "EditRemoteWake", &fEditRemoteWake);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         fEditRemoteWake = true;    /* NB: On by default! */
     else
@@ -1193,10 +1205,10 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
     }
 
     /*
-     * Disable remote wakeup capability, see @bugref{9839}. This is done on 
-     * a device/configuration level, no need to dig too deep through the descriptors. 
-     * On most backends, we can't perform a real selective suspend, and more importantly 
-     * can't receive a remote wake notification. If a guest suspends the device and waits 
+     * Disable remote wakeup capability, see @bugref{9839}. This is done on
+     * a device/configuration level, no need to dig too deep through the descriptors.
+     * On most backends, we can't perform a real selective suspend, and more importantly
+     * can't receive a remote wake notification. If a guest suspends the device and waits
      * for a remote wake, the device is effectively dead.
      */
     if (fEditRemoteWake)
@@ -1206,9 +1218,12 @@ static DECLCALLBACK(int) usbProxyConstruct(PPDMUSBINS pUsbIns, int iInstance, PC
         {
             Log(("usb-proxy: pProxyDev=%s configuration %d with bmAttr=%02X\n",
                  pUsbIns->pszName, paCfgs[iCfg].Core.bmAttributes, iCfg));
+            if (paCfgs[iCfg].Core.bmAttributes & RT_BIT(5))
+            {
                  paCfgs[iCfg].Core.bmAttributes = paCfgs[iCfg].Core.bmAttributes & ~RT_BIT(5); /* Remote wakeup. */
-            fEdited = true;
-            LogRel(("VUSB: Disabled '%s' remote wakeup for configuration %d\n", pUsbIns->pszName, iCfg));
+                 fEdited = true;
+                 LogRel(("VUSB: Disabled '%s' remote wakeup for configuration %d\n", pUsbIns->pszName, iCfg));
+            }
         }
     }
 
