@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2010-2020 Oracle Corporation
+ * Copyright (C) 2010-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #ifndef FEQT_INCLUDED_SRC_runtime_normal_UIMachineWindowNormal_h
@@ -29,8 +39,7 @@ class CMediumAttachment;
 class UIIndicatorsPool;
 class UIAction;
 
-/** UIMachineWindow reimplementation,
-  * providing GUI with machine-window for the normal mode. */
+/** UIMachineWindow subclass used as normal machine window implementation. */
 class UIMachineWindowNormal : public UIMachineWindow
 {
     Q_OBJECT;
@@ -40,7 +49,7 @@ signals:
     /** Notifies about geometry change. */
     void sigGeometryChange(const QRect &rect);
 
-protected:
+public:
 
     /** Constructor, passes @a pMachineLogic and @a uScreenId to the UIMachineWindow constructor. */
     UIMachineWindowNormal(UIMachineLogic *pMachineLogic, ulong uScreenId);
@@ -97,15 +106,19 @@ private:
 #endif /* !VBOX_WS_MAC */
     /** Prepare status-bar routine. */
     void prepareStatusBar();
+    /** Prepare notification-center routine. */
+    void prepareNotificationCenter();
     /** Prepare visual-state routine. */
     void prepareVisualState();
     /** Load settings routine. */
     void loadSettings();
 
-    /** Save settings routine. */
-    void saveSettings();
     /** Cleanup visual-state routine. */
     void cleanupVisualState();
+    /** Cleanup notification-center routine. */
+    void cleanupNotificationCenter();
+    /** Cleanup status-bar routine. */
+    void cleanupStatusBar();
     /** Cleanup session connections routine. */
     void cleanupSessionConnections();
 
@@ -113,11 +126,12 @@ private:
     void showInNecessaryMode();
 
     /** Restores cached window geometry. */
-    virtual void restoreCachedGeometry() /* override */;
+    virtual void restoreCachedGeometry() RT_OVERRIDE;
 
     /** Performs window geometry normalization according to guest-size and host's available geometry.
-      * @param  fAdjustPosition  Determines whether is it necessary to adjust position as well. */
-    virtual void normalizeGeometry(bool fAdjustPosition) /* override */;
+      * @param  fAdjustPosition        Determines whether is it necessary to adjust position as well.
+      * @param  fResizeToGuestDisplay  Determines whether is it necessary to resize the window to fit to guest display size. */
+    virtual void normalizeGeometry(bool fAdjustPosition, bool fResizeToGuestDisplay) RT_OVERRIDE;
 
     /** Common update routine. */
     void updateAppearanceOf(int aElement);
@@ -137,11 +151,9 @@ private:
     UIIndicatorsPool *m_pIndicatorsPool;
 
     /** Holds the current window geometry. */
-    QRect m_normalGeometry;
-
-    /** Factory support. */
-    friend class UIMachineWindow;
+    QRect  m_geometry;
+    /** Holds the geometry save timer ID. */
+    int  m_iGeometrySaveTimerId;
 };
 
 #endif /* !FEQT_INCLUDED_SRC_runtime_normal_UIMachineWindowNormal_h */
-
