@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2011-2020 Oracle Corporation
+ * Copyright (C) 2011-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -108,26 +118,6 @@
 /*********************************************************************************************************************************
 *   Defined Constants And Macros                                                                                                 *
 *********************************************************************************************************************************/
-/*
- * Note: Must match the VID & PID in the USB driver .inf file!!
- */
-/*
-  BusQueryDeviceID USB\Vid_80EE&Pid_CAFE
-  BusQueryInstanceID 2
-  BusQueryHardwareIDs USB\Vid_80EE&Pid_CAFE&Rev_0100
-  BusQueryHardwareIDs USB\Vid_80EE&Pid_CAFE
-  BusQueryCompatibleIDs USB\Class_ff&SubClass_00&Prot_00
-  BusQueryCompatibleIDs USB\Class_ff&SubClass_00
-  BusQueryCompatibleIDs USB\Class_ff
-*/
-
-#define szBusQueryDeviceId       L"USB\\Vid_80EE&Pid_CAFE"
-#define szBusQueryHardwareIDs    L"USB\\Vid_80EE&Pid_CAFE&Rev_0100\0USB\\Vid_80EE&Pid_CAFE\0\0"
-#define szBusQueryCompatibleIDs  L"USB\\Class_ff&SubClass_00&Prot_00\0USB\\Class_ff&SubClass_00\0USB\\Class_ff\0\0"
-
-#define szDeviceTextDescription          L"VirtualBox USB"
-
-
 #define VBOXUSBMON_MEMTAG 'MUBV'
 
 
@@ -180,6 +170,24 @@ typedef struct VBOXUSBMONGLOBALS
 *********************************************************************************************************************************/
 static VBOXUSBMONGLOBALS g_VBoxUsbMonGlobals;
 
+/*
+ * Note: Must match the VID & PID in the USB driver .inf file!!
+ */
+/*
+  BusQueryDeviceID USB\Vid_80EE&Pid_CAFE
+  BusQueryInstanceID 2
+  BusQueryHardwareIDs USB\Vid_80EE&Pid_CAFE&Rev_0100
+  BusQueryHardwareIDs USB\Vid_80EE&Pid_CAFE
+  BusQueryCompatibleIDs USB\Class_ff&SubClass_00&Prot_00
+  BusQueryCompatibleIDs USB\Class_ff&SubClass_00
+  BusQueryCompatibleIDs USB\Class_ff
+*/
+
+static WCHAR const g_szBusQueryDeviceId[]      = L"USB\\Vid_80EE&Pid_CAFE";
+static WCHAR const g_szBusQueryHardwareIDs[]   = L"USB\\Vid_80EE&Pid_CAFE&Rev_0100\0USB\\Vid_80EE&Pid_CAFE\0\0";
+static WCHAR const g_szBusQueryCompatibleIDs[] = L"USB\\Class_ff&SubClass_00&Prot_00\0USB\\Class_ff&SubClass_00\0USB\\Class_ff\0\0";
+static WCHAR const g_szDeviceTextDescription[] = L"VirtualBox USB";
+
 
 
 PVOID VBoxUsbMonMemAlloc(SIZE_T cbBytes)
@@ -297,10 +305,8 @@ NTSTATUS VBoxUsbMonQueryBusRelations(PDEVICE_OBJECT pDevObj, PFILE_OBJECT pFileO
     {
         PDEVICE_RELATIONS pRel = (PDEVICE_RELATIONS)IoStatus.Information;
         LOG(("pRel = %p", pRel));
-        if (VALID_PTR(pRel))
-        {
+        if (RT_VALID_PTR(pRel))
             *pDevRelations = pRel;
-        }
         else
         {
             WARN(("Invalid pointer %p", pRel));
@@ -371,12 +377,12 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
             if (pIoStatus->Status == STATUS_SUCCESS)
             {
                 WCHAR *pId = (WCHAR *)pIoStatus->Information;
-                if (VALID_PTR(pId))
+                if (RT_VALID_PTR(pId))
                 {
                     KIRQL Iqrl = KeGetCurrentIrql();
                     /* IRQL should be always passive here */
                     ASSERT_WARN(Iqrl == PASSIVE_LEVEL, ("irql is not PASSIVE"));
-                    switch(pSl->Parameters.QueryDeviceText.DeviceTextType)
+                    switch (pSl->Parameters.QueryDeviceText.DeviceTextType)
                     {
                         case DeviceTextLocationInformation:
                             LOG(("DeviceTextLocationInformation"));
@@ -389,17 +395,13 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
                             if (VBoxUsbFltPdoIsFiltered(pDevObj))
                             {
                                 LOG(("PDO (0x%p) is filtered", pDevObj));
-                                WCHAR *pId = (WCHAR *)ExAllocatePool(PagedPool, sizeof(szDeviceTextDescription));
-                                if (!pId)
-                                {
-                                    AssertFailed();
-                                    break;
-                                }
-                                memcpy(pId, szDeviceTextDescription, sizeof(szDeviceTextDescription));
+                                WCHAR *pId2 = (WCHAR *)ExAllocatePool(PagedPool, sizeof(g_szDeviceTextDescription));
+                                AssertBreak(pId2);
+                                memcpy(pId2, g_szDeviceTextDescription, sizeof(g_szDeviceTextDescription));
                                 LOG(("NEW szDeviceTextDescription"));
-                                LOG_STRW(pId);
+                                LOG_STRW(pId2);
                                 ExFreePool((PVOID)pIoStatus->Information);
-                                pIoStatus->Information = (ULONG_PTR)pId;
+                                pIoStatus->Information = (ULONG_PTR)pId2;
                             }
                             else
                             {
@@ -426,7 +428,7 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
 #ifdef VBOX_USB_WITH_VERBOSE_LOGGING
                 WCHAR *pTmp;
 #endif
-                if (VALID_PTR(pId))
+                if (RT_VALID_PTR(pId))
                 {
                     KIRQL Iqrl = KeGetCurrentIrql();
                     /* IRQL should be always passive here */
@@ -442,7 +444,7 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
                         case BusQueryDeviceID:
                         {
                             LOG(("BusQueryDeviceID"));
-                            pId = (WCHAR *)ExAllocatePool(PagedPool, sizeof(szBusQueryDeviceId));
+                            pId = (WCHAR *)ExAllocatePool(PagedPool, sizeof(g_szBusQueryDeviceId));
                             if (!pId)
                             {
                                 WARN(("ExAllocatePool failed"));
@@ -467,7 +469,7 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
 
                             LOG(("PDO (0x%p) is filtered", pDevObj));
                             ExFreePool((PVOID)pIoStatus->Information);
-                            memcpy(pId, szBusQueryDeviceId, sizeof(szBusQueryDeviceId));
+                            memcpy(pId, g_szBusQueryDeviceId, sizeof(g_szBusQueryDeviceId));
                             pIoStatus->Information = (ULONG_PTR)pId;
                             break;
                         }
@@ -482,7 +484,7 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
                             pId++;
                         }
 #endif
-                        pId = (WCHAR *)ExAllocatePool(PagedPool, sizeof(szBusQueryHardwareIDs));
+                        pId = (WCHAR *)ExAllocatePool(PagedPool, sizeof(g_szBusQueryHardwareIDs));
                         if (!pId)
                         {
                             WARN(("ExAllocatePool failed"));
@@ -507,7 +509,7 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
 
                         LOG(("PDO (0x%p) is filtered", pDevObj));
 
-                        memcpy(pId, szBusQueryHardwareIDs, sizeof(szBusQueryHardwareIDs));
+                        memcpy(pId, g_szBusQueryHardwareIDs, sizeof(g_szBusQueryHardwareIDs));
 #ifdef VBOX_USB_WITH_VERBOSE_LOGGING
                         LOG(("NEW BusQueryHardwareIDs"));
                         pTmp = pId;
@@ -536,13 +538,13 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
                         if (VBoxUsbFltPdoIsFiltered(pDevObj))
                         {
                             LOG(("PDO (0x%p) is filtered", pDevObj));
-                            pId = (WCHAR *)ExAllocatePool(PagedPool, sizeof(szBusQueryCompatibleIDs));
+                            pId = (WCHAR *)ExAllocatePool(PagedPool, sizeof(g_szBusQueryCompatibleIDs));
                             if (!pId)
                             {
                                 WARN(("ExAllocatePool failed"));
                                 break;
                             }
-                            memcpy(pId, szBusQueryCompatibleIDs, sizeof(szBusQueryCompatibleIDs));
+                            memcpy(pId, g_szBusQueryCompatibleIDs, sizeof(g_szBusQueryCompatibleIDs));
 #ifdef VBOX_USB_WITH_VERBOSE_LOGGING
                             LOG(("NEW BusQueryCompatibleIDs"));
                             pTmp = pId;
@@ -587,7 +589,7 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
                     {
                         PDEVICE_RELATIONS pRel = (PDEVICE_RELATIONS)pIoStatus->Information;
                         LOG(("pRel = %p", pRel));
-                        if (VALID_PTR(pRel))
+                        if (RT_VALID_PTR(pRel))
                         {
                             for (unsigned i=0;i<pRel->Count;i++)
                             {
@@ -620,7 +622,7 @@ static NTSTATUS vboxUsbMonHandlePnPIoctl(PDEVICE_OBJECT pDevObj, PIO_STACK_LOCAT
             if (pIoStatus->Status == STATUS_SUCCESS)
             {
                 PDEVICE_CAPABILITIES pCaps = pSl->Parameters.DeviceCapabilities.Capabilities;
-                if (VALID_PTR(pCaps))
+                if (RT_VALID_PTR(pCaps))
                 {
                     LOG(("Caps.SilentInstall  = %d", pCaps->SilentInstall));
                     LOG(("Caps.UniqueID       = %d", pCaps->UniqueID ));
