@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2008-2020 Oracle Corporation
+ * Copyright (C) 2008-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #ifndef MAIN_INCLUDED_Global_h
@@ -45,6 +55,10 @@
 #define VBOXOSHINT_TFRESET              RT_BIT(13)
 #define VBOXOSHINT_USB3                 RT_BIT(14)
 #define VBOXOSHINT_X2APIC               RT_BIT(15)
+#define VBOXOSHINT_EFI_SECUREBOOT       RT_BIT(16)
+#define VBOXOSHINT_TPM                  RT_BIT(17)
+#define VBOXOSHINT_TPM2                 RT_BIT(18)
+#define VBOXOSHINT_WDDM_GRAPHICS        RT_BIT(19)
 
 /** The VBoxVRDP kludge extension pack name.
  *
@@ -53,6 +67,13 @@
  */
 #define VBOXVRDP_KLUDGE_EXTPACK_NAME    "Built-in-VBoxVRDP"
 
+/** The VBoxPuelCrypto kludge extension pack name.
+ *
+ * This is not a valid extension pack name (dashes are not allowed), and
+ * hence will not conflict with real extension packs.
+ */
+#define VBOXPUELCRYPTO_KLUDGE_EXTPACK_NAME    "Built-in-VBoxPuelCrypto"
+
 /**
  * Contains global static definitions that can be referenced by all COM classes
  * regardless of the apartment.
@@ -82,6 +103,7 @@ public:
         const StorageControllerType_T  hdStorageControllerType;
         const StorageBus_T             hdStorageBusType;
         const ChipsetType_T            chipsetType;
+        const IommuType_T              iommuType;
         const AudioControllerType_T    audioControllerType;
         const AudioCodecType_T         audioCodecType;
     };
@@ -141,7 +163,10 @@ public:
     }
 
     /**
-     * Stringify a machine state.
+     * Stringify a machine state - translated.
+     *
+     * Drop the Global:: prefix and include StringifyEnums.h for an untranslated
+     * version of this method.
      *
      * @returns Pointer to a read only string.
      * @param   aState      Valid machine state.
@@ -149,7 +174,10 @@ public:
     static const char *stringifyMachineState(MachineState_T aState);
 
     /**
-     * Stringify a session state.
+     * Stringify a session state - translated.
+     *
+     * Drop the Global:: prefix and include StringifyEnums.h for an untranslated
+     * version of this method.
      *
      * @returns Pointer to a read only string.
      * @param   aState      Valid session state.
@@ -159,27 +187,49 @@ public:
     /**
      * Stringify a device type.
      *
+     * Drop the Global:: prefix and include StringifyEnums.h for an untranslated
+     * version of this method.
+     *
      * @returns Pointer to a read only string.
      * @param   aType       The device type.
      */
     static const char *stringifyDeviceType(DeviceType_T aType);
 
-    /**
-     * Stringify a reason.
-     *
-     * @returns Pointer to a read only string.
-     * @param   aReason     The reason code.
-     */
-    static const char *stringifyReason(Reason_T aReason);
-
     /**
      * Stringify a storage controller type.
      *
+     * Drop the Global:: prefix and include StringifyEnums.h for an untranslated
+     * version of this method.
+     *
      * @returns Pointer to a read only string.
      * @param   aType       The storage controller type.
      */
     static const char *stringifyStorageControllerType(StorageControllerType_T aType);
 
+#if 0 /* unused */
+    /**
+     * Stringify a storage bus type.
+     *
+     * Drop the Global:: prefix and include StringifyEnums.h for an untranslated
+     * version of this method.
+     *
+     * @returns Pointer to a read only string.
+     * @param   aBus        The storage bus type.
+     */
+    static const char *stringifyStorageBus(StorageBus_T aBus);
+
+    /**
+     * Stringify a reason.
+     *
+     * Drop the Global:: prefix and include StringifyEnums.h for an untranslated
+     * version of this method.
+     *
+     * @returns Pointer to a read only string.
+     * @param   aReason     The reason code.
+     */
+    static const char *stringifyReason(Reason_T aReason);
+#endif
+
     /**
      * Try convert a COM status code to a VirtualBox status code (VBox/err.h).
      *
