@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2011-2020 Oracle Corporation
+ * Copyright (C) 2011-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #ifndef MAIN_INCLUDED_GuestCtrlImplPrivate_h
@@ -734,6 +744,44 @@ struct GuestFileOpenInfo
         , mCreationMode(0)
         , mfOpenEx(0) { }
 
+    /**
+     * Validates a file open info.
+     *
+     * @returns \c true if valid, \c false if not.
+     */
+    bool IsValid(void) const
+    {
+        if (mfOpenEx) /** @todo Open flags not implemented yet. */
+            return false;
+
+        switch (mOpenAction)
+        {
+            case FileOpenAction_OpenExisting:
+                break;
+            case FileOpenAction_OpenOrCreate:
+                break;
+            case FileOpenAction_CreateNew:
+                break;
+            case FileOpenAction_CreateOrReplace:
+                break;
+            case FileOpenAction_OpenExistingTruncated:
+            {
+                if (   mAccessMode == FileAccessMode_ReadOnly
+                    || mAccessMode == FileAccessMode_AppendOnly
+                    || mAccessMode == FileAccessMode_AppendRead)
+                    return false;
+                break;
+            }
+            case FileOpenAction_AppendOrCreate: /* Deprecated, do not use. */
+                break;
+            default:
+                AssertFailedReturn(false);
+                break;
+        }
+
+        return true; /** @todo Do we need more checks here? */
+    }
+
     /** The filename. */
     Utf8Str                 mFilename;
     /** The file access mode. */
@@ -775,6 +823,7 @@ struct GuestFsObjData
     /** @name Helper functions to extract the data from a certin VBoxService tool's guest stream block.
      * @{ */
     int FromLs(const GuestProcessStreamBlock &strmBlk, bool fLong);
+    int FromRm(const GuestProcessStreamBlock &strmBlk);
     int FromStat(const GuestProcessStreamBlock &strmBlk);
     int FromMkTemp(const GuestProcessStreamBlock &strmBlk);
     /** @}  */
@@ -887,8 +936,8 @@ class GuestProcessStreamValue
 public:
 
     GuestProcessStreamValue(void) { }
-    GuestProcessStreamValue(const char *pszValue, size_t cwcValue = RTSTR_MAX)
-        : mValue(pszValue, cwcValue) {}
+    GuestProcessStreamValue(const char *pszValue)
+        : mValue(pszValue) {}
 
     GuestProcessStreamValue(const GuestProcessStreamValue& aThat)
            : mValue(aThat.mValue) { }
@@ -910,22 +959,13 @@ typedef std::map < Utf8Str, GuestProcessStreamValue > GuestCtrlStreamPairMap;
 typedef std::map < Utf8Str, GuestProcessStreamValue >::iterator GuestCtrlStreamPairMapIter;
 typedef std::map < Utf8Str, GuestProcessStreamValue >::const_iterator GuestCtrlStreamPairMapIterConst;
 
-class GuestProcessStream;
-
 /**
  * Class representing a block of stream pairs (key=value). Each block in a raw guest
  * output stream is separated by "\0\0", each pair is separated by "\0". The overall
  * end of a guest stream is marked by "\0\0\0\0".
- *
- * An empty stream block will be treated as being incomplete.
- *
- * Only used for the busybox-like toolbox commands within VBoxService.
- * Deprecated, do not use anymore.
  */
 class GuestProcessStreamBlock
 {
-    friend class GuestProcessStream;
-
 public:
 
     GuestProcessStreamBlock(void);
@@ -949,19 +989,13 @@ public:
     uint32_t    GetUInt32(const char *pszKey, uint32_t uDefault = 0) const;
     int32_t     GetInt32(const char *pszKey, int32_t iDefault = 0) const;
 
-    bool        IsComplete(void) const { return !m_mapPairs.empty() && m_fComplete; }
-    bool        IsEmpty(void) const { return m_mapPairs.empty(); }
+    bool        IsEmpty(void) { return mPairs.empty(); }
 
-    int         SetValueEx(const char *pszKey, size_t cwcKey, const char *pszValue, size_t cwcValue, bool fOverwrite = false);
     int         SetValue(const char *pszKey, const char *pszValue);
 
 protected:
 
-    /** Wheter the stream block is marked as complete.
-     *  An empty stream block is considered as incomplete. */
-    bool                   m_fComplete;
-    /** Map of stream pairs this block contains.*/
-    GuestCtrlStreamPairMap m_mapPairs;
+    GuestCtrlStreamPairMap mPairs;
 };
 
 /** Vector containing multiple allocated stream pair objects. */
@@ -969,23 +1003,6 @@ typedef std::vector< GuestProcessStreamBlock > GuestCtrlStreamObjects;
 typedef std::vector< GuestProcessStreamBlock >::iterator GuestCtrlStreamObjectsIter;
 typedef std::vector< GuestProcessStreamBlock >::const_iterator GuestCtrlStreamObjectsIterConst;
 
-/** Defines a single terminator as a single char. */
-#define GUESTTOOLBOX_STRM_TERM                      '\0'
-/** Defines a single terminator as a string. */
-#define GUESTTOOLBOX_STRM_TERM_STR                  "\0"
-/** Defines the termination sequence for a single key/value pair. */
-#define GUESTTOOLBOX_STRM_TERM_PAIR_STR             GUESTTOOLBOX_STRM_TERM_STR
-/** Defines the termination sequence for a single stream block. */
-#define GUESTTOOLBOX_STRM_TERM_BLOCK_STR            GUESTTOOLBOX_STRM_TERM_STR GUESTTOOLBOX_STRM_TERM_STR
-/** Defines the termination sequence for the stream. */
-#define GUESTTOOLBOX_STRM_TERM_STREAM_STR           GUESTTOOLBOX_STRM_TERM_STR GUESTTOOLBOX_STRM_TERM_STR GUESTTOOLBOX_STRM_TERM_STR GUESTTOOLBOX_STRM_TERM_STR
-/** Defines how many consequtive terminators a key/value pair has. */
-#define GUESTTOOLBOX_STRM_PAIR_TERM_CNT             1
-/** Defines how many consequtive terminators a stream block has. */
-#define GUESTTOOLBOX_STRM_BLK_TERM_CNT              2
-/** Defines how many consequtive terminators a stream has. */
-#define GUESTTOOLBOX_STRM_TERM_CNT                  4
-
 /**
  * Class for parsing machine-readable guest process output by VBoxService'
  * toolbox commands ("vbox_ls", "vbox_stat" etc), aka "guest stream".
@@ -1009,11 +1026,9 @@ public:
     void Dump(const char *pszFile);
 #endif
 
-    size_t GetOffset(void) const { return m_offBuf; }
+    size_t GetOffset() { return m_offBuffer; }
 
-    size_t GetSize(void) const { return m_cbUsed; }
-
-    size_t GetBlocks(void) const { return m_cBlocks; }
+    size_t GetSize() { return m_cbUsed; }
 
     int ParseBlock(GuestProcessStreamBlock &streamBlock);
 
@@ -1027,11 +1042,9 @@ protected:
     /** Currently used size at m_offBuffer. */
     size_t m_cbUsed;
     /** Current byte offset within the internal stream buffer. */
-    size_t m_offBuf;
+    size_t m_offBuffer;
     /** Internal stream buffer. */
     BYTE  *m_pbBuffer;
-    /** How many completed stream blocks already were processed. */
-    size_t m_cBlocks;
 };
 
 class Guest;
@@ -1275,17 +1288,13 @@ public:
     int unregisterWaitEvent(GuestWaitEvent *pEvent);
     int waitForEvent(GuestWaitEvent *pEvent, uint32_t uTimeoutMS, VBoxEventType_T *pType, IEvent **ppEvent);
 
-#ifndef VBOX_GUESTCTRL_TEST_CASE
-    static Utf8Str getErrorAsString(const Utf8Str &strAction, const GuestErrorInfo& guestErrorInfo);
-    static Utf8Str getErrorAsString(const GuestErrorInfo &guestErrorInfo);
-    HRESULT setErrorExternal(VirtualBoxBase *pInterface, const Utf8Str &strAction, const GuestErrorInfo &guestErrorInfo);
-#endif
-
 public:
 
     static FsObjType_T fileModeToFsObjType(RTFMODE fMode);
     static const char *fsObjTypeToStr(FsObjType_T enmType);
     static const char *pathStyleToStr(PathStyle_T enmPathStyle);
+    static Utf8Str getErrorAsString(const Utf8Str &strAction, const GuestErrorInfo& guestErrorInfo);
+    static Utf8Str getErrorAsString(const GuestErrorInfo &guestErrorInfo);
 
 protected:
 
