@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2010-2017 Oracle Corporation
+ * Copyright (C) 2010-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -34,6 +44,7 @@
 #include <iprt/buildconfig.h>
 #include <iprt/ctype.h>
 #include <iprt/dir.h>
+#include <iprt/err.h>
 #include <iprt/file.h>
 #include <iprt/getopt.h>
 #include <iprt/initterm.h>
@@ -62,6 +73,7 @@
 #define RTZIPTARCMD_OPT_FORMAT              1009
 #define RTZIPTARCMD_OPT_READ_AHEAD          1010
 #define RTZIPTARCMD_OPT_USE_PUSH_FILE       1011
+#define RTZIPTARCMD_OPT_NO_RECURSION        1012
 
 /** File format. */
 typedef enum RTZIPTARCMDFORMAT
@@ -72,7 +84,9 @@ typedef enum RTZIPTARCMDFORMAT
     /** TAR.  */
     RTZIPTARCMDFORMAT_TAR,
     /** XAR.  */
-    RTZIPTARCMDFORMAT_XAR
+    RTZIPTARCMDFORMAT_XAR,
+    /** CPIO. */
+    RTZIPTARCMDFORMAT_CPIO
 } RTZIPTARCMDFORMAT;
 
 
@@ -109,6 +123,8 @@ typedef struct RTZIPTARCMDOPS
     bool            fReadAhead;
     /** Use RTVfsFsStrmPushFile instead of RTVfsFsStrmAdd for files. */
     bool            fUsePushFile;
+    /** Whether to handle directories recursively or not. Defaults to \c true. */
+    bool            fRecursive;
     /** The compressor/decompressor method to employ (0, z or j). */
     char            chZipper;
 
@@ -151,6 +167,9 @@ typedef struct RTZIPTARCMDOPS
 /** Pointer to the IPRT tar options. */
 typedef RTZIPTARCMDOPS *PRTZIPTARCMDOPS;
 
+/** The size of the directory entry buffer we're using. */
+#define RTZIPTARCMD_DIRENTRY_BUF_SIZE (sizeof(RTDIRENTRYEX) + RTPATH_MAX)
+
 /**
  * Callback used by rtZipTarDoWithMembers
  *
@@ -185,6 +204,50 @@ static bool rtZipTarCmdIsNameInArray(const char *pszName, const char * const *pa
 }
 
 
+/**
+ * Queries information about a VFS object.
+ *
+ * @returns VBox status code.
+ * @param   pszSpec             VFS object spec to use.
+ * @param   paObjInfo           Where to store the queried object information.
+ *                              Must at least provide 3 structs, namely for UNIX, UNIX_OWNER and UNIX_GROUP attributes.
+ * @param   cObjInfo            Number of objection information structs handed in.
+ */
+static int rtZipTarCmdQueryObjInfo(const char *pszSpec, PRTFSOBJINFO paObjInfo, unsigned cObjInfo)
+{
+    AssertPtrReturn(paObjInfo, VERR_INVALID_POINTER);
+    AssertReturn(cObjInfo >= 3, VERR_INVALID_PARAMETER);
+
+    RTERRINFOSTATIC ErrInfo;
+    uint32_t        offError;
+    int rc = RTVfsChainQueryInfo(pszSpec, &paObjInfo[0], RTFSOBJATTRADD_UNIX, RTPATH_F_ON_LINK,
+                                 &offError, RTErrInfoInitStatic(&ErrInfo));
+    if (RT_SUCCESS(rc))
+    {
+        rc = RTVfsChainQueryInfo(pszSpec, &paObjInfo[1], RTFSOBJATTRADD_UNIX_OWNER, RTPATH_F_ON_LINK,
+                                 &offError, RTErrInfoInitStatic(&ErrInfo));
+        if (RT_SUCCESS(rc))
+        {
+            rc = RTVfsChainQueryInfo(pszSpec, &paObjInfo[2], RTFSOBJATTRADD_UNIX_GROUP, RTPATH_F_ON_LINK,
+                                     &offError, RTErrInfoInitStatic(&ErrInfo));
+            if (RT_FAILURE(rc))
+                RT_BZERO(&paObjInfo[2], sizeof(RTFSOBJINFO));
+        }
+        else
+        {
+            RT_BZERO(&paObjInfo[1], sizeof(RTFSOBJINFO));
+            RT_BZERO(&paObjInfo[2], sizeof(RTFSOBJINFO));
+        }
+
+        rc = VINF_SUCCESS; /* aObjInfo[1] + aObjInfo[2] are optional. */
+    }
+    else
+        RTVfsChainMsgError("RTVfsChainQueryInfo", pszSpec, rc, offError, &ErrInfo.Core);
+
+    return rc;
+}
+
+
 /**
  * Archives a file.
  *
@@ -252,7 +315,131 @@ static RTEXITCODE rtZipTarCmdArchiveFile(PRTZIPTARCMDOPS pOpts, RTVFSFSSTREAM hV
 
 
 /**
- * Archives a directory recursively .
+ * Sub-directory helper for creating archives.
+ *
+ * @returns RTEXITCODE_SUCCESS or RTEXITCODE_FAILURE + printed message.
+ * @param   pOpts           The options.
+ * @param   hVfsFss         The TAR filesystem stream handle.
+ * @param   pszSrc          The directory path or VFS spec.  We append to the
+ *                          buffer as we decend.
+ * @param   cchSrc          The length of the input.
+ * @param   paObjInfo[3]    Array of three FS object info structures.  The first
+ *                          one is always filled with RTFSOBJATTRADD_UNIX info.
+ *                          The next two may contain owner and group names if
+ *                          available.  The three buffers can be reused.
+ * @param   pszDst          The name to archive it the under.  We append to the
+ *                          buffer as we decend.
+ * @param   cchDst          The length of the input.
+ * @param   pDirEntry       Directory entry to use for the directory to handle.
+ * @param   pErrInfo        Error info buffer (saves stack space).
+ */
+static RTEXITCODE rtZipTarCmdArchiveDirSub(PRTZIPTARCMDOPS pOpts, RTVFSFSSTREAM hVfsFss,
+                                           char *pszSrc, size_t cchSrc, RTFSOBJINFO paObjInfo[3],
+                                           char pszDst[RTPATH_MAX], size_t cchDst, PRTDIRENTRYEX pDirEntry,
+                                           PRTERRINFOSTATIC pErrInfo)
+{
+    if (pOpts->fVerbose)
+        RTPrintf("%s\n", pszDst);
+
+    uint32_t offError;
+    RTVFSDIR hVfsIoDir;
+    int rc = RTVfsChainOpenDir(pszSrc, 0 /*fFlags*/,
+                               &hVfsIoDir, &offError, RTErrInfoInitStatic(pErrInfo));
+    if (RT_FAILURE(rc))
+        return RTVfsChainMsgErrorExitFailure("RTVfsChainOpenDir", pszSrc, rc, offError, &pErrInfo->Core);
+
+    /* Make sure we've got some room in the path, to save us extra work further down. */
+    if (cchSrc + 3 >= RTPATH_MAX)
+        return RTMsgErrorExitFailure("Source path too long: '%s'\n", pszSrc);
+
+    /* Ensure we've got a trailing slash (there is space for it see above). */
+    if (!RTPATH_IS_SEP(pszSrc[cchSrc - 1]))
+    {
+        pszSrc[cchSrc++] = RTPATH_SLASH;
+        pszSrc[cchSrc]   = '\0';
+    }
+
+    /* Ditto for destination. */
+    if (cchDst + 3 >= RTPATH_MAX)
+        return RTMsgErrorExitFailure("Destination path too long: '%s'\n", pszDst);
+
+    if (!RTPATH_IS_SEP(pszDst[cchDst - 1]))
+    {
+        pszDst[cchDst++] = RTPATH_SLASH;
+        pszDst[cchDst]   = '\0';
+    }
+
+    /*
+     * Process the files and subdirs.
+     */
+    for (;;)
+    {
+        size_t cbDirEntry = RTZIPTARCMD_DIRENTRY_BUF_SIZE;
+        rc = RTVfsDirReadEx(hVfsIoDir, pDirEntry, &cbDirEntry, RTFSOBJATTRADD_UNIX);
+        if (RT_FAILURE(rc))
+            break;
+
+        /* Check length. */
+        if (pDirEntry->cbName + cchSrc + 3 >= RTPATH_MAX)
+        {
+            rc = VERR_BUFFER_OVERFLOW;
+            break;
+        }
+
+        switch (pDirEntry->Info.Attr.fMode & RTFS_TYPE_MASK)
+        {
+            case RTFS_TYPE_DIRECTORY:
+            {
+                if (RTDirEntryExIsStdDotLink(pDirEntry))
+                    continue;
+
+                if (!pOpts->fRecursive)
+                    continue;
+
+                memcpy(&pszSrc[cchSrc], pDirEntry->szName, pDirEntry->cbName + 1);
+                if (RT_SUCCESS(rc))
+                {
+                    memcpy(&pszDst[cchDst], pDirEntry->szName, pDirEntry->cbName + 1);
+                    rc = rtZipTarCmdArchiveDirSub(pOpts, hVfsFss, pszSrc, cchSrc + pDirEntry->cbName, paObjInfo,
+                                                  pszDst, cchDst + pDirEntry->cbName, pDirEntry, pErrInfo);
+                }
+
+                break;
+            }
+
+            case RTFS_TYPE_FILE:
+            {
+                memcpy(&pszSrc[cchSrc], pDirEntry->szName, pDirEntry->cbName + 1);
+                rc = rtZipTarCmdQueryObjInfo(pszSrc, paObjInfo, 3 /* cObjInfo */);
+                if (RT_SUCCESS(rc))
+                {
+                    memcpy(&pszDst[cchDst], pDirEntry->szName, pDirEntry->cbName + 1);
+                    rc = rtZipTarCmdArchiveFile(pOpts, hVfsFss, pszSrc, paObjInfo, pszDst, pErrInfo);
+                }
+                break;
+            }
+
+            default:
+            {
+                if (pOpts->fVerbose)
+                    RTPrintf("Warning: File system type %#x for '%s' not implemented yet, sorry! Skipping ...\n",
+                             pDirEntry->Info.Attr.fMode & RTFS_TYPE_MASK, pDirEntry->szName);
+                break;
+            }
+        }
+    }
+
+    RTVfsDirRelease(hVfsIoDir);
+
+    if (rc != VERR_NO_MORE_FILES)
+        return RTMsgErrorExitFailure("RTVfsDirReadEx failed unexpectedly!");
+
+    return RTEXITCODE_SUCCESS;
+}
+
+
+/**
+ * Archives a directory recursively.
  *
  * @returns RTEXITCODE_SUCCESS or RTEXITCODE_FAILURE + printed message.
  * @param   pOpts           The options.
@@ -273,10 +460,21 @@ static RTEXITCODE rtZipTarCmdArchiveDir(PRTZIPTARCMDOPS pOpts, RTVFSFSSTREAM hVf
                                         RTFSOBJINFO paObjInfo[3], char pszDst[RTPATH_MAX], size_t cchDst,
                                         PRTERRINFOSTATIC pErrInfo)
 {
-    RT_NOREF(pOpts, hVfsFss, pszSrc, cchSrc, paObjInfo, pszDst, cchDst, pErrInfo);
-    return RTMsgErrorExitFailure("Adding directories has not yet been implemented! Sorry.");
-}
+    RT_NOREF(cchSrc);
 
+    char szSrcAbs[RTPATH_MAX];
+    int rc = RTPathAbs(pszSrc, szSrcAbs, sizeof(szSrcAbs));
+    if (RT_FAILURE(rc))
+        return RTMsgErrorExitFailure("RTPathAbs failed on '%s': %Rrc\n", pszSrc, rc);
+
+    union
+    {
+        uint8_t         abPadding[RTZIPTARCMD_DIRENTRY_BUF_SIZE];
+        RTDIRENTRYEX    DirEntry;
+    } uBuf;
+
+    return rtZipTarCmdArchiveDirSub(pOpts, hVfsFss, szSrcAbs, strlen(szSrcAbs), paObjInfo, pszDst, cchDst, &uBuf.DirEntry, pErrInfo);
+}
 
 
 /**
@@ -489,28 +687,11 @@ static RTEXITCODE rtZipTarCreate(PRTZIPTARCMDOPS pOpts)
                     /*
                      * What kind of object is this and what affiliations does it have?
                      */
-                    RTERRINFOSTATIC ErrInfo;
-                    uint32_t        offError;
-                    RTFSOBJINFO     aObjInfo[3];
-                    rc = RTVfsChainQueryInfo(szSrc, &aObjInfo[0], RTFSOBJATTRADD_UNIX, RTPATH_F_ON_LINK,
-                                             &offError, RTErrInfoInitStatic(&ErrInfo));
+                    RTFSOBJINFO aObjInfo[3];
+                    rc = rtZipTarCmdQueryObjInfo(szSrc, aObjInfo, RT_ELEMENTS(aObjInfo));
                     if (RT_SUCCESS(rc))
                     {
-
-                        rc = RTVfsChainQueryInfo(szSrc, &aObjInfo[1], RTFSOBJATTRADD_UNIX_OWNER, RTPATH_F_ON_LINK,
-                                                 &offError, RTErrInfoInitStatic(&ErrInfo));
-                        if (RT_SUCCESS(rc))
-                        {
-                            rc = RTVfsChainQueryInfo(szSrc, &aObjInfo[2], RTFSOBJATTRADD_UNIX_GROUP, RTPATH_F_ON_LINK,
-                                                     &offError, RTErrInfoInitStatic(&ErrInfo));
-                            if (RT_FAILURE(rc))
-                                RT_ZERO(aObjInfo[2]);
-                        }
-                        else
-                        {
-                            RT_ZERO(aObjInfo[1]);
-                            RT_ZERO(aObjInfo[2]);
-                        }
+                        RTERRINFOSTATIC ErrInfo;
 
                         /*
                          * Process on an object type basis.
@@ -537,7 +718,7 @@ static RTEXITCODE rtZipTarCreate(PRTZIPTARCMDOPS pOpts)
                             rcExit = rcExit2;
                     }
                     else
-                        rcExit = RTVfsChainMsgErrorExitFailure("RTVfsChainQueryInfo", pszFile, rc, offError, &ErrInfo.Core);
+                        rcExit = RTMsgErrorExitFailure("querying object information for '%s' failed (%s)", szSrc, pszFile);
                 }
                 else
                     rcExit = RTMsgErrorExitFailure("archived file name is too long, skipping '%s' (%s)", pszDst, pszFile);
@@ -648,6 +829,8 @@ static RTEXITCODE rtZipTarCmdOpenInputArchive(PRTZIPTARCMDOPS pOpts, PRTVFSFSSTR
 #else
         rc = VERR_NOT_SUPPORTED;
 #endif
+    else if (pOpts->enmFormat == RTZIPTARCMDFORMAT_CPIO)
+        rc = RTZipCpioFsStreamFromIoStream(hVfsIos, 0/*fFlags*/, phVfsFss);
     else /** @todo make RTZipTarFsStreamFromIoStream fail if not tar file! */
         rc = RTZipTarFsStreamFromIoStream(hVfsIos, 0/*fFlags*/, phVfsFss);
     RTVfsIoStrmRelease(hVfsIos);
@@ -1436,9 +1619,11 @@ static void rtZipTarUsage(const char *pszProgName)
              "                  ustar (tar POSIX.1-1988), "
              "                  pax (tar POSIX.1-2001),\n"
              "                  xar\n"
-             "        Note! Because XAR/TAR detection isn't implemented yet, it\n"
+             "                  cpio\n"
+             "        Note! Because XAR/TAR/CPIO detection isn't implemented yet, it\n"
              "              is necessary to specifcy --format=xar when reading a\n"
-             "              XAR file.  Otherwise this option is only for creation.\n"
+             "              XAR file or --format=cpio for a CPIO file.\n"
+             "              Otherwise this option is only for creation.\n"
              "\n");
     RTPrintf("IPRT Options:\n"
              "    --prefix <dir-prefix>                 (-A, -c, -d, -r, -u)\n"
@@ -1504,6 +1689,7 @@ RTDECL(RTEXITCODE) RTZipTarCmd(unsigned cArgs, char **papszArgs)
         { "--utc",                  RTZIPTARCMD_OPT_UTC,                RTGETOPT_REQ_NOTHING },
         { "--sparse",               'S',                                RTGETOPT_REQ_NOTHING },
         { "--format",               RTZIPTARCMD_OPT_FORMAT,             RTGETOPT_REQ_STRING },
+        { "--no-recursion",         RTZIPTARCMD_OPT_NO_RECURSION,       RTGETOPT_REQ_NOTHING },
 
         /* IPRT extensions */
         { "--prefix",               RTZIPTARCMD_OPT_PREFIX,             RTGETOPT_REQ_STRING },
@@ -1538,6 +1724,7 @@ RTDECL(RTEXITCODE) RTZipTarCmd(unsigned cArgs, char **papszArgs)
     }
 #endif
     Opts.enmTarFormat = RTZIPTARFORMAT_DEFAULT;
+    Opts.fRecursive   = true; /* Recursion is implicit unless otherwise specified. */
 
     RTGETOPTUNION   ValueUnion;
     while (   (rc = RTGetOpt(&GetState, &ValueUnion)) != 0
@@ -1628,6 +1815,10 @@ RTDECL(RTEXITCODE) RTZipTarCmd(unsigned cArgs, char **papszArgs)
                 Opts.fDisplayUtc = true;
                 break;
 
+            case RTZIPTARCMD_OPT_NO_RECURSION:
+                Opts.fRecursive = false;
+                break;
+
             /* GNU */
             case 'S':
                 Opts.fTarCreate |= RTZIPTAR_C_SPARSE;
@@ -1685,6 +1876,8 @@ RTDECL(RTEXITCODE) RTZipTarCmd(unsigned cArgs, char **papszArgs)
                 }
                 else if (!strcmp(ValueUnion.psz, "xar"))
                     Opts.enmFormat    = RTZIPTARCMDFORMAT_XAR;
+                else if (!strcmp(ValueUnion.psz, "cpio"))
+                    Opts.enmFormat    = RTZIPTARCMDFORMAT_CPIO;
                 else
                     return RTMsgErrorExit(RTEXITCODE_SYNTAX, "Unknown archive format: '%s'", ValueUnion.psz);
                 break;
@@ -1719,6 +1912,9 @@ RTDECL(RTEXITCODE) RTZipTarCmd(unsigned cArgs, char **papszArgs)
         Opts.cFiles     = cArgs - GetState.iNext + 1;
     }
 
+    if (!Opts.pszFile)
+        return RTMsgErrorExitFailure("No archive specified");
+
     /*
      * Post proceess the options.
      */
