@@ -6,24 +6,34 @@
 /*
  * Contributed by knut st. osmundsen, Andriy Gapon, Arto Huusko.
  *
- * Copyright (C) 2007-2020 Oracle Corporation
+ * Copyright (C) 2007-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  * --------------------------------------------------------------------
  *
  * Copyright (c) 2007 knut st. osmundsen <bird-src-spam@anduin.net>
@@ -188,11 +198,10 @@ DECLHIDDEN(int) rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
 }
 
 static int rtR0MemObjNetBSDAllocHelper(PRTR0MEMOBJNETBSD pMemNetBSD, size_t cb, bool fExecutable,
-                                         paddr_t VmPhysAddrHigh, bool fContiguous)
+                                       paddr_t VmPhysAddrHigh, bool fContiguous)
 {
     /* Virtual space first */
-    vaddr_t virt = uvm_km_alloc(kernel_map, cb, 0,
-            UVM_KMF_VAONLY | UVM_KMF_WAITVA | UVM_KMF_CANFAIL);
+    vaddr_t virt = uvm_km_alloc(kernel_map, cb, 0, UVM_KMF_VAONLY | UVM_KMF_WAITVA | UVM_KMF_CANFAIL);
     if (virt == 0)
         return VERR_NO_MEMORY;
 
@@ -201,11 +210,10 @@ static int rtR0MemObjNetBSDAllocHelper(PRTR0MEMOBJNETBSD pMemNetBSD, size_t cb,
     int nsegs = fContiguous ? 1 : INT_MAX;
 
     /* Physical pages */
-    if (uvm_pglistalloc(cb, 0, VmPhysAddrHigh,
-            PAGE_SIZE, 0, rlist, nsegs, 1) != 0)
+    if (uvm_pglistalloc(cb, 0, VmPhysAddrHigh, PAGE_SIZE, 0, rlist, nsegs, 1) != 0)
     {
         uvm_km_free(kernel_map, virt, cb, UVM_KMF_VAONLY);
-        return VERR_NO_MEMORY;
+        return VERR_NO_MEMORY; /** @todo inaccurate status code */
     }
 
     /* Map */
@@ -220,6 +228,7 @@ static int rtR0MemObjNetBSDAllocHelper(PRTR0MEMOBJNETBSD pMemNetBSD, size_t cb,
         virt2 += PAGE_SIZE;
     }
 
+    pMemNetBSD->Core.fFlags |= RTR0MEMOBJ_FLAGS_UNINITIALIZED_AT_ALLOC; /*?*/
     pMemNetBSD->Core.pv = (void *)virt;
     if (fContiguous)
     {
@@ -230,28 +239,26 @@ static int rtR0MemObjNetBSDAllocHelper(PRTR0MEMOBJNETBSD pMemNetBSD, size_t cb,
 }
 
 
-DECLHIDDEN(int) rtR0MemObjNativeAllocPage(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
+DECLHIDDEN(int) rtR0MemObjNativeAllocPage(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable, const char *pszTag)
 {
-    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD),
-                                                                       RTR0MEMOBJTYPE_PAGE, NULL, cb);
-    if (!pMemNetBSD)
-        return VERR_NO_MEMORY;
-
-    void *pvMem = kmem_alloc(cb, KM_SLEEP);
-    if (RT_UNLIKELY(!pvMem))
+    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_PAGE, NULL, cb, pszTag);
+    if (pMemNetBSD)
     {
+        void *pvMem = kmem_alloc(cb, KM_SLEEP);
+        if (pvMem)
+        {
+            if (fExecutable)
+                pmap_protect(pmap_kernel(), (vaddr_t)pvMem, (vaddr_t)pvMem + cb, VM_PROT_READ | VM_PROT_WRITE | VM_PROT_EXECUTE);
+
+            pMemNetBSD->Core.fFlags |= RTR0MEMOBJ_FLAGS_UNINITIALIZED_AT_ALLOC;
+            pMemNetBSD->Core.pv      = pvMem;
+            *ppMem = &pMemNetBSD->Core;
+            return VINF_SUCCESS;
+        }
         rtR0MemObjDelete(&pMemNetBSD->Core);
         return VERR_NO_PAGE_MEMORY;
     }
-    if (fExecutable)
-    {
-        pmap_protect(pmap_kernel(), (vaddr_t)pvMem, ((vaddr_t)pvMem) + cb,
-                VM_PROT_READ | VM_PROT_WRITE | VM_PROT_EXECUTE);
-    }
-
-    pMemNetBSD->Core.pv = pvMem;
-    *ppMem = &pMemNetBSD->Core;
-    return VINF_SUCCESS;
+    return VERR_NO_MEMORY;
 }
 
 
@@ -262,152 +269,144 @@ DECLHIDDEN(int) rtR0MemObjNativeAllocLarge(PPRTR0MEMOBJINTERNAL ppMem, size_t cb
 }
 
 
-DECLHIDDEN(int) rtR0MemObjNativeAllocLow(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
+DECLHIDDEN(int) rtR0MemObjNativeAllocLow(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable, const char *pszTag)
 {
-    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD),
-                                                                       RTR0MEMOBJTYPE_LOW, NULL, cb);
-    if (!pMemNetBSD)
-        return VERR_NO_MEMORY;
-
-    int rc = rtR0MemObjNetBSDAllocHelper(pMemNetBSD, cb, fExecutable, _4G - 1, false);
-    if (rc)
+    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_LOW, NULL, cb, pszTag);
+    if (pMemNetBSD)
     {
+        int rc = rtR0MemObjNetBSDAllocHelper(pMemNetBSD, cb, fExecutable, _4G - 1, false /*fContiguous*/);
+        if (RT_SUCCESS(rc))
+        {
+            *ppMem = &pMemNetBSD->Core;
+            return VINF_SUCCESS;
+        }
         rtR0MemObjDelete(&pMemNetBSD->Core);
         return rc;
     }
-
-    *ppMem = &pMemNetBSD->Core;
-    return VINF_SUCCESS;
+    return VERR_NO_MEMORY;
 }
 
 
-DECLHIDDEN(int) rtR0MemObjNativeAllocCont(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
+DECLHIDDEN(int) rtR0MemObjNativeAllocCont(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable, const char *pszTag)
 {
-    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD),
-                                                                       RTR0MEMOBJTYPE_CONT, NULL, cb);
-    if (!pMemNetBSD)
-        return VERR_NO_MEMORY;
-
-    int rc = rtR0MemObjNetBSDAllocHelper(pMemNetBSD, cb, fExecutable, _4G - 1, true);
-    if (rc)
+    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_CONT, NULL, cb, pszTag);
+    if (pMemNetBSD)
     {
+        int rc = rtR0MemObjNetBSDAllocHelper(pMemNetBSD, cb, fExecutable, _4G - 1, true /*fContiguous*/);
+        if (RT_SUCCESS(rc))
+        {
+            *ppMem = &pMemNetBSD->Core;
+            return VINF_SUCCESS;
+        }
         rtR0MemObjDelete(&pMemNetBSD->Core);
         return rc;
     }
-
-    *ppMem = &pMemNetBSD->Core;
-    return VINF_SUCCESS;
+    return VERR_NO_MEMORY;
 }
 
 
-static int rtR0MemObjNetBSDAllocPhysPages(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJTYPE enmType,
-                                           size_t cb,
-                                           RTHCPHYS PhysHighest, size_t uAlignment,
-                                           bool fContiguous)
+static int rtR0MemObjNetBSDAllocPhysPages(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJTYPE enmType, size_t cb,
+                                          RTHCPHYS PhysHighest, size_t uAlignment, bool fContiguous, const char *pszTag)
 {
-    paddr_t VmPhysAddrHigh;
-
     /* create the object. */
-    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD),
-                                                                       enmType, NULL, cb);
-    if (!pMemNetBSD)
-        return VERR_NO_MEMORY;
-
-    if (PhysHighest != NIL_RTHCPHYS)
-        VmPhysAddrHigh = PhysHighest;
-    else
-        VmPhysAddrHigh = ~(paddr_t)0;
-
-    int nsegs = fContiguous ? 1 : INT_MAX;
-
-    int error = uvm_pglistalloc(cb, 0, VmPhysAddrHigh, uAlignment, 0, &pMemNetBSD->pglist, nsegs, 1);
-    if (error)
+    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), enmType, NULL, cb, pszTag);
+    if (pMemNetBSD)
     {
+        paddr_t const VmPhysAddrHigh = PhysHighest != NIL_RTHCPHYS ? PhysHighest : ~(paddr_t)0;
+        int const     nsegs          = fContiguous ? 1 : INT_MAX;
+        int rc = uvm_pglistalloc(cb, 0, VmPhysAddrHigh, uAlignment, 0, &pMemNetBSD->pglist, nsegs, 1);
+        if (!rc)
+        {
+            pMemNetBSD->Core.fFlags |= RTR0MEMOBJ_FLAGS_UNINITIALIZED_AT_ALLOC; /*?*/
+            if (fContiguous)
+            {
+                Assert(enmType == RTR0MEMOBJTYPE_PHYS);
+                const struct vm_page * const pg = TAILQ_FIRST(&pMemNetBSD->pglist);
+                pMemNetBSD->Core.u.Phys.PhysBase = VM_PAGE_TO_PHYS(pg);
+                pMemNetBSD->Core.u.Phys.fAllocated = true;
+            }
+            *ppMem = &pMemNetBSD->Core;
+            return VINF_SUCCESS;
+        }
         rtR0MemObjDelete(&pMemNetBSD->Core);
-        return VERR_NO_MEMORY;
+        return VERR_NO_PAGE_MEMORY;
     }
-
-    if (fContiguous)
-    {
-        Assert(enmType == RTR0MEMOBJTYPE_PHYS);
-        const struct vm_page * const pg = TAILQ_FIRST(&pMemNetBSD->pglist);
-        pMemNetBSD->Core.u.Phys.PhysBase = VM_PAGE_TO_PHYS(pg);
-        pMemNetBSD->Core.u.Phys.fAllocated = true;
-    }
-    *ppMem = &pMemNetBSD->Core;
-
-    return VINF_SUCCESS;
+    return VERR_NO_MEMORY;
 }
 
 
-DECLHIDDEN(int) rtR0MemObjNativeAllocPhys(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, RTHCPHYS PhysHighest, size_t uAlignment)
+DECLHIDDEN(int) rtR0MemObjNativeAllocPhys(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, RTHCPHYS PhysHighest, size_t uAlignment,
+                                          const char *pszTag)
 {
-    return rtR0MemObjNetBSDAllocPhysPages(ppMem, RTR0MEMOBJTYPE_PHYS, cb, PhysHighest, uAlignment, true);
+    return rtR0MemObjNetBSDAllocPhysPages(ppMem, RTR0MEMOBJTYPE_PHYS, cb, PhysHighest, uAlignment, true, pszTag);
 }
 
 
-DECLHIDDEN(int) rtR0MemObjNativeAllocPhysNC(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, RTHCPHYS PhysHighest)
+DECLHIDDEN(int) rtR0MemObjNativeAllocPhysNC(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, RTHCPHYS PhysHighest, const char *pszTag)
 {
-    return rtR0MemObjNetBSDAllocPhysPages(ppMem, RTR0MEMOBJTYPE_PHYS_NC, cb, PhysHighest, PAGE_SIZE, false);
+    return rtR0MemObjNetBSDAllocPhysPages(ppMem, RTR0MEMOBJTYPE_PHYS_NC, cb, PhysHighest, PAGE_SIZE, false, pszTag);
 }
 
 
-DECLHIDDEN(int) rtR0MemObjNativeEnterPhys(PPRTR0MEMOBJINTERNAL ppMem, RTHCPHYS Phys, size_t cb, uint32_t uCachePolicy)
+DECLHIDDEN(int) rtR0MemObjNativeEnterPhys(PPRTR0MEMOBJINTERNAL ppMem, RTHCPHYS Phys, size_t cb, uint32_t uCachePolicy,
+                                          const char *pszTag)
 {
     AssertReturn(uCachePolicy == RTMEM_CACHE_POLICY_DONT_CARE, VERR_NOT_SUPPORTED);
 
     /* create the object. */
-    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_PHYS, NULL, cb);
-    if (!pMemNetBSD)
-        return VERR_NO_MEMORY;
-
-    /* there is no allocation here, it needs to be mapped somewhere first. */
-    pMemNetBSD->Core.u.Phys.fAllocated = false;
-    pMemNetBSD->Core.u.Phys.PhysBase = Phys;
-    pMemNetBSD->Core.u.Phys.uCachePolicy = uCachePolicy;
-    TAILQ_INIT(&pMemNetBSD->pglist);
-    *ppMem = &pMemNetBSD->Core;
-    return VINF_SUCCESS;
-}
-
-
-DECLHIDDEN(int) rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t cb, uint32_t fAccess, RTR0PROCESS R0Process)
-{
-    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_LOCK, (void *)R3Ptr, cb);
-    if (!pMemNetBSD)
-        return VERR_NO_MEMORY;
-
-    int rc = uvm_map_pageable(
-            &((struct proc *)R0Process)->p_vmspace->vm_map,
-            R3Ptr,
-            R3Ptr + cb,
-            0, 0);
-    if (rc)
+    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_PHYS, NULL, cb, pszTag);
+    if (pMemNetBSD)
     {
-        rtR0MemObjDelete(&pMemNetBSD->Core);
-        return VERR_NO_MEMORY;
+        /* there is no allocation here, it needs to be mapped somewhere first. */
+        pMemNetBSD->Core.u.Phys.fAllocated = false;
+        pMemNetBSD->Core.u.Phys.PhysBase = Phys;
+        pMemNetBSD->Core.u.Phys.uCachePolicy = uCachePolicy;
+        TAILQ_INIT(&pMemNetBSD->pglist);
+        *ppMem = &pMemNetBSD->Core;
+        return VINF_SUCCESS;
     }
-
-    pMemNetBSD->Core.u.Lock.R0Process = R0Process;
-    *ppMem = &pMemNetBSD->Core;
-    return VINF_SUCCESS;
+    return VERR_NO_MEMORY;
 }
 
 
-DECLHIDDEN(int) rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb, uint32_t fAccess)
+DECLHIDDEN(int) rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t cb, uint32_t fAccess,
+                                         RTR0PROCESS R0Process, const char *pszTag)
+{
+    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_LOCK,
+                                                                    (void *)R3Ptr, cb, pszTag);
+    if (pMemNetBSD)
+    {
+        int rc = uvm_map_pageable(&((struct proc *)R0Process)->p_vmspace->vm_map, R3Ptr, R3Ptr + cb,
+                                  0 /*new_pageable*/, 0 /*lockflags*/);
+        if (!rc)
+        {
+            pMemNetBSD->Core.u.Lock.R0Process = R0Process;
+            *ppMem = &pMemNetBSD->Core;
+            return VINF_SUCCESS;
+        }
+        rtR0MemObjDelete(&pMemNetBSD->Core);
+        return VERR_LOCK_FAILED;
+    }
+    return VERR_NO_MEMORY;
+}
+
+
+DECLHIDDEN(int) rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb, uint32_t fAccess, const char *pszTag)
 {
     /* Kernel memory (always?) wired; all memory allocated by vbox code is? */
-    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_LOCK, pv, cb);
-    if (!pMemNetBSD)
-        return VERR_NO_MEMORY;
-
-    pMemNetBSD->Core.u.Lock.R0Process = NIL_RTR0PROCESS;
-    pMemNetBSD->Core.pv = pv;
-    *ppMem = &pMemNetBSD->Core;
-    return VINF_SUCCESS;
+    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_LOCK, pv, cb, pszTag);
+    if (pMemNetBSD)
+    {
+        pMemNetBSD->Core.u.Lock.R0Process = NIL_RTR0PROCESS;
+        pMemNetBSD->Core.pv = pv;
+        *ppMem = &pMemNetBSD->Core;
+        return VINF_SUCCESS;
+    }
+    return VERR_NO_MEMORY;
 }
 
-DECLHIDDEN(int) rtR0MemObjNativeReserveKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pvFixed, size_t cb, size_t uAlignment)
+DECLHIDDEN(int) rtR0MemObjNativeReserveKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pvFixed, size_t cb, size_t uAlignment,
+                                              const char *pszTag)
 {
     if (pvFixed != (void *)-1)
     {
@@ -416,34 +415,36 @@ DECLHIDDEN(int) rtR0MemObjNativeReserveKernel(PPRTR0MEMOBJINTERNAL ppMem, void *
         return VERR_NOT_SUPPORTED;
     }
 
-    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_RES_VIRT, NULL, cb);
-    if (!pMemNetBSD)
-        return VERR_NO_MEMORY;
-
-    vaddr_t virt = uvm_km_alloc(kernel_map, cb, uAlignment,
-            UVM_KMF_VAONLY | UVM_KMF_WAITVA | UVM_KMF_CANFAIL);
-    if (virt == 0)
+    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_RES_VIRT,
+                                                                    NULL, cb, pszTag);
+    if (pMemNetBSD)
     {
+        vaddr_t virt = uvm_km_alloc(kernel_map, cb, uAlignment, UVM_KMF_VAONLY | UVM_KMF_WAITVA | UVM_KMF_CANFAIL);
+        if (virt != 0)
+        {
+            pMemNetBSD->Core.u.ResVirt.R0Process = NIL_RTR0PROCESS;
+            pMemNetBSD->Core.pv = (void *)virt;
+            *ppMem = &pMemNetBSD->Core;
+            return VINF_SUCCESS;
+        }
         rtR0MemObjDelete(&pMemNetBSD->Core);
         return VERR_NO_MEMORY;
     }
-
-    pMemNetBSD->Core.u.ResVirt.R0Process = NIL_RTR0PROCESS;
-    pMemNetBSD->Core.pv = (void *)virt;
-    *ppMem = &pMemNetBSD->Core;
-    return VINF_SUCCESS;
+    return VERR_NO_MEMORY;
 }
 
 
-DECLHIDDEN(int) rtR0MemObjNativeReserveUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3PtrFixed, size_t cb, size_t uAlignment, RTR0PROCESS R0Process)
+DECLHIDDEN(int) rtR0MemObjNativeReserveUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3PtrFixed, size_t cb, size_t uAlignment,
+                                            RTR0PROCESS R0Process, const char *pszTag)
 {
+    RT_NOREF(ppMem, R3PtrFixed, cb, uAlignment, R0Process, pszTag);
     printf("NativeReserveUser\n");
     return VERR_NOT_SUPPORTED;
 }
 
 
 DECLHIDDEN(int) rtR0MemObjNativeMapKernel(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJ pMemToMap, void *pvFixed, size_t uAlignment,
-                                          unsigned fProt, size_t offSub, size_t cbSub)
+                                          unsigned fProt, size_t offSub, size_t cbSub, const char *pszTag)
 {
     if (pvFixed != (void *)-1)
     {
@@ -461,52 +462,50 @@ DECLHIDDEN(int) rtR0MemObjNativeMapKernel(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJ
     }
     size_t sz = cbSub > 0 ? cbSub : pMemNetBSD0->Core.cb;
 
-    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_MAPPING, NULL, sz);
+    PRTR0MEMOBJNETBSD pMemNetBSD = (PRTR0MEMOBJNETBSD)rtR0MemObjNew(sizeof(*pMemNetBSD), RTR0MEMOBJTYPE_MAPPING, NULL, sz, pszTag);
 
-    vaddr_t virt = uvm_km_alloc(kernel_map, sz, uAlignment,
-            UVM_KMF_VAONLY | UVM_KMF_WAITVA | UVM_KMF_CANFAIL);
-    if (virt == 0)
+    vaddr_t virt = uvm_km_alloc(kernel_map, sz, uAlignment, UVM_KMF_VAONLY | UVM_KMF_WAITVA | UVM_KMF_CANFAIL);
+    if (virt != 0)
     {
-        rtR0MemObjDelete(&pMemNetBSD->Core);
-        return VERR_NO_MEMORY;
-    }
+        vm_prot_t prot = 0;
+        if (fProt & RTMEM_PROT_READ)
+            prot |= VM_PROT_READ;
+        if (fProt & RTMEM_PROT_WRITE)
+            prot |= VM_PROT_WRITE;
+        if (fProt & RTMEM_PROT_EXEC)
+            prot |= VM_PROT_EXECUTE;
 
-    vm_prot_t prot = 0;
-
-    if ((fProt & RTMEM_PROT_READ) == RTMEM_PROT_READ)
-        prot |= VM_PROT_READ;
-    if ((fProt & RTMEM_PROT_WRITE) == RTMEM_PROT_WRITE)
-        prot |= VM_PROT_WRITE;
-    if ((fProt & RTMEM_PROT_EXEC) == RTMEM_PROT_EXEC)
-        prot |= VM_PROT_EXECUTE;
-
-    struct vm_page *page;
-    vaddr_t virt2 = virt;
-    size_t map_pos = 0;
-    TAILQ_FOREACH(page, &pMemNetBSD0->pglist, pageq.queue)
-    {
-        if (map_pos >= offSub)
+        struct vm_page *page;
+        vaddr_t virt2 = virt;
+        size_t map_pos = 0;
+        TAILQ_FOREACH(page, &pMemNetBSD0->pglist, pageq.queue)
         {
-            if (cbSub > 0 && (map_pos >= offSub + cbSub))
-                break;
+            if (map_pos >= offSub)
+            {
+                if (cbSub > 0 && (map_pos >= offSub + cbSub))
+                    break;
 
-            pmap_kenter_pa(virt2, VM_PAGE_TO_PHYS(page), prot, 0);
-            virt2 += PAGE_SIZE;
+                pmap_kenter_pa(virt2, VM_PAGE_TO_PHYS(page), prot, 0);
+                virt2 += PAGE_SIZE;
+            }
+            map_pos += PAGE_SIZE;
         }
-        map_pos += PAGE_SIZE;
+
+        pMemNetBSD->Core.pv = (void *)virt;
+        pMemNetBSD->Core.u.Mapping.R0Process = NIL_RTR0PROCESS;
+        *ppMem = &pMemNetBSD->Core;
+        return VINF_SUCCESS;
     }
 
-    pMemNetBSD->Core.pv = (void *)virt;
-    pMemNetBSD->Core.u.Mapping.R0Process = NIL_RTR0PROCESS;
-    *ppMem = &pMemNetBSD->Core;
-
-    return VINF_SUCCESS;
+    rtR0MemObjDelete(&pMemNetBSD->Core);
+    return VERR_NO_MEMORY;
 }
 
 
 DECLHIDDEN(int) rtR0MemObjNativeMapUser(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJ pMemToMap, RTR3PTR R3PtrFixed, size_t uAlignment,
-                                        unsigned fProt, RTR0PROCESS R0Process, size_t offSub, size_t cbSub)
+                                        unsigned fProt, RTR0PROCESS R0Process, size_t offSub, size_t cbSub, const char *pszTag)
 {
+    RT_NOREF(ppMem, pMemToMap, R3PtrFixed, uAlignment, fProt, R0Process, offSub, cbSub, pszTag);
     printf("NativeMapUser\n");
     return VERR_NOT_SUPPORTED;
 }
@@ -514,25 +513,23 @@ DECLHIDDEN(int) rtR0MemObjNativeMapUser(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJ p
 
 DECLHIDDEN(int) rtR0MemObjNativeProtect(PRTR0MEMOBJINTERNAL pMem, size_t offSub, size_t cbSub, uint32_t fProt)
 {
-    vm_prot_t          ProtectionFlags = 0;
-    vaddr_t        AddrStart       = (vaddr_t)pMem->pv + offSub;
-    vm_map_t           pVmMap          = rtR0MemObjNetBSDGetMap(pMem);
-
-    if (!pVmMap)
-        return VERR_NOT_SUPPORTED;
-
-    if ((fProt & RTMEM_PROT_READ) == RTMEM_PROT_READ)
-        ProtectionFlags |= UVM_PROT_R;
-    if ((fProt & RTMEM_PROT_WRITE) == RTMEM_PROT_WRITE)
-        ProtectionFlags |= UVM_PROT_W;
-    if ((fProt & RTMEM_PROT_EXEC) == RTMEM_PROT_EXEC)
-        ProtectionFlags |= UVM_PROT_X;
-
-    int error = uvm_map_protect(pVmMap, AddrStart, AddrStart + cbSub,
-        ProtectionFlags, 0);
-    if (!error)
-        return VINF_SUCCESS;
+    vm_map_t const pVmMap = rtR0MemObjNetBSDGetMap(pMem);
+    if (pVmMap)
+    {
+        vaddr_t const   AddrStart       = (vaddr_t)pMem->pv + offSub;
+        vm_prot_t       ProtectionFlags = 0;
+        if (fProt & RTMEM_PROT_READ)
+            ProtectionFlags |= UVM_PROT_R;
+        if (fProt & RTMEM_PROT_WRITE)
+            ProtectionFlags |= UVM_PROT_W;
+        if (fProt & RTMEM_PROT_EXEC)
+            ProtectionFlags |= UVM_PROT_X;
 
+        int rc = uvm_map_protect(pVmMap, AddrStart, AddrStart + cbSub, ProtectionFlags, 0);
+        if (!rc)
+            return VINF_SUCCESS;
+        return RTErrConvertFromErrno(rc);
+    }
     return VERR_NOT_SUPPORTED;
 }
 
