@@ -7,26 +7,36 @@ Test Manager - Database Interface.
 
 __copyright__ = \
 """
-Copyright (C) 2012-2017 Oracle Corporation
+Copyright (C) 2012-2022 Oracle and/or its affiliates.
 
-This file is part of VirtualBox Open Source Edition (OSE), as
-available from http://www.virtualbox.org. This file is free software;
-you can redistribute it and/or modify it under the terms of the GNU
-General Public License (GPL) as published by the Free Software
-Foundation, in version 2 as it comes in the "COPYING" file of the
-VirtualBox OSE distribution. VirtualBox OSE is distributed in the
-hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+This file is part of VirtualBox base platform packages, as
+available from https://www.virtualbox.org.
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation, in version 3 of the
+License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, see <https://www.gnu.org/licenses>.
 
 The contents of this file may alternatively be used under the terms
 of the Common Development and Distribution License Version 1.0
-(CDDL) only, as it comes in the "COPYING.CDDL" file of the
-VirtualBox OSE distribution, in which case the provisions of the
+(CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+in the VirtualBox distribution, in which case the provisions of the
 CDDL are applicable instead of those of the GPL.
 
 You may elect to license modified versions of this file under the
 terms and conditions of either the GPL or the CDDL or both.
+
+SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
 """
-__version__ = "$Revision: 118412 $"
+__version__ = "$Revision: 154728 $"
 
 
 # Standard python imports.
@@ -44,6 +54,9 @@ from testmanager                        import config;
 if sys.version_info[0] < 3:
     psycopg2.extensions.register_type(psycopg2.extensions.UNICODE);
     psycopg2.extensions.register_type(psycopg2.extensions.UNICODEARRAY);
+else:
+    unicode = str;  # pylint: disable=redefined-builtin,invalid-name
+
 
 
 def isDbTimestampInfinity(tsValue):
@@ -62,7 +75,7 @@ def isDbTimestamp(oValue):
     if utils.isString(oValue):
         ## @todo detect strings as well.
         return False;
-    return getattr(oValue, 'pydatetime', None) != None;
+    return getattr(oValue, 'pydatetime', None) is not None;
 
 def dbTimestampToDatetime(oValue):
     """
@@ -71,7 +84,7 @@ def dbTimestampToDatetime(oValue):
     if isinstance(oValue, datetime.datetime):
         return oValue;
     if utils.isString(oValue):
-        raise Exception('TODO');
+        return utils.parseIsoTimestamp(oValue);
     return oValue.pydatetime();
 
 def dbTimestampToZuluDatetime(oValue):
@@ -100,6 +113,18 @@ def dbTimestampPythonNow():
     """
     return dbTimestampToZuluDatetime(datetime.datetime.utcnow());
 
+def dbOneTickIntervalString():
+    """
+    Returns the interval string for one tick.
+
+    Mogrify the return value into the SQL:
+        "... %s::INTERVAL ..."
+    or
+        "INTERVAL %s"
+    The completed SQL will contain the necessary ticks.
+    """
+    return '1 microsecond';
+
 def dbTimestampMinusOneTick(oValue):
     """
     Returns a new timestamp that's one tick before the given one.
@@ -107,6 +132,13 @@ def dbTimestampMinusOneTick(oValue):
     oValue = dbTimestampToZuluDatetime(oValue);
     return oValue - datetime.timedelta(microseconds = 1);
 
+def dbTimestampPlusOneTick(oValue):
+    """
+    Returns a new timestamp that's one tick after the given one.
+    """
+    oValue = dbTimestampToZuluDatetime(oValue);
+    return oValue + datetime.timedelta(microseconds = 1);
+
 def isDbInterval(oValue):
     """
     Checks if oValue is a DB interval object.
@@ -123,7 +155,7 @@ class TMDatabaseIntegrityException(Exception):
     Do NOT use directly, only thru TMDatabaseConnection.integrityException.
     Otherwise, we won't be able to log the issue.
     """
-    pass;
+    pass;                               # pylint: disable=unnecessary-pass
 
 
 class TMDatabaseCursor(object):
@@ -210,13 +242,13 @@ class TMDatabaseConnection(object):
             dArgs['host'] = config.g_ksDatabaseAddress;
         if config.g_ksDatabasePort is not None:
             dArgs['port'] = config.g_ksDatabasePort;
-        self._oConn             = psycopg2.connect(**dArgs); # pylint: disable=W0142
+        self._oConn             = psycopg2.connect(**dArgs); # pylint: disable=star-args
         self._oConn.set_client_encoding('UTF-8');
         self._oCursor           = self._oConn.cursor();
         self._oExplainConn      = None;
         self._oExplainCursor    = None;
         if config.g_kfWebUiSqlTraceExplain and config.g_kfWebUiSqlTrace:
-            self._oExplainConn  = psycopg2.connect(**dArgs); # pylint: disable=W0142
+            self._oExplainConn  = psycopg2.connect(**dArgs); # pylint: disable=star-args
             self._oExplainConn.set_client_encoding('UTF-8');
             self._oExplainCursor = self._oExplainConn.cursor();
         self._fTransaction      = False;
@@ -236,7 +268,7 @@ class TMDatabaseConnection(object):
             oSrvGlue.registerDebugInfoCallback(self.debugInfoCallback);
 
         # Object caches (used by database logic classes).
-        self.ddCaches = dict();
+        self.ddCaches = {};
 
     def isAutoCommitting(self):
         """ Work around missing autocommit attribute in older versions."""
@@ -341,12 +373,12 @@ class TMDatabaseConnection(object):
         if aoArgs is not None:
             sBound = oCursor.mogrify(unicode(sOperation), aoArgs);
         elif sOperation.find('%') < 0:
-            sBound = oCursor.mogrify(unicode(sOperation), list());
+            sBound = oCursor.mogrify(unicode(sOperation), []);
         else:
             sBound = unicode(sOperation);
 
         if sys.version_info[0] >= 3 and not isinstance(sBound, str):
-            sBound = sBound.decode('utf-8');
+            sBound = sBound.decode('utf-8'); # pylint: disable=redefined-variable-type
 
         aasExplain = None;
         if self._oExplainCursor is not None and not sBound.startswith('DROP'):
@@ -393,7 +425,7 @@ class TMDatabaseConnection(object):
         collect data for traceback.
         """
         if aoArgs is None:
-            aoArgs = list();
+            aoArgs = [];
 
         nsStart = utils.timestampNano();
         try:
@@ -555,7 +587,7 @@ class TMDatabaseConnection(object):
         """ Returns the cache dictionary for this data type. """
         dRet = self.ddCaches.get(sType, None);
         if dRet is None:
-            dRet = dict();
+            dRet = {};
             self.ddCaches[sType] = dRet;
         return dRet;
 
@@ -619,10 +651,10 @@ class TMDatabaseConnection(object):
         for aEntry in self._aoTraceBack:
             iEntry += 1;
             sDebug += ' <tr>\n' \
-                      '  <td align="right">%s</td>\n' \
-                      '  <td align="right">%s</td>\n' \
-                      '  <td align="right">%s</td>\n' \
-                      '  <td align="right">%s</td>\n' \
+                      '  <td>%s</td>\n' \
+                      '  <td>%s</td>\n' \
+                      '  <td>%s</td>\n' \
+                      '  <td>%s</td>\n' \
                       '  <td><pre>%s</pre></td>\n' \
                       '  <td>%s</td>\n' \
                       ' </tr>\n' \
@@ -697,7 +729,7 @@ class TMDatabaseConnection(object):
                 dArgs['host'] = config.g_ksDatabaseAddress;
             if config.g_ksDatabasePort is not None:
                 dArgs['port'] = config.g_ksDatabasePort;
-            self._oExplainConn  = psycopg2.connect(**dArgs); # pylint: disable=W0142
+            self._oExplainConn  = psycopg2.connect(**dArgs); # pylint: disable=star-args
             self._oExplainCursor = self._oExplainConn.cursor();
         return True;
 
